# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from weaviate.proto.v1.v6300.v1 import aggregate_pb2 as v1_dot_aggregate__pb2
from weaviate.proto.v1.v6300.v1 import batch_delete_pb2 as v1_dot_batch__delete__pb2
from weaviate.proto.v1.v6300.v1 import batch_pb2 as v1_dot_batch__pb2
from weaviate.proto.v1.v6300.v1 import search_get_pb2 as v1_dot_search__get__pb2
from weaviate.proto.v1.v6300.v1 import tenants_pb2 as v1_dot_tenants__pb2

GRPC_GENERATED_VERSION = '1.72.1'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in v1/weaviate_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class WeaviateStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Search = channel.unary_unary(
                '/weaviate.v1.Weaviate/Search',
                request_serializer=v1_dot_search__get__pb2.SearchRequest.SerializeToString,
                response_deserializer=v1_dot_search__get__pb2.SearchReply.FromString,
                _registered_method=True)
        self.BatchObjects = channel.unary_unary(
                '/weaviate.v1.Weaviate/BatchObjects',
                request_serializer=v1_dot_batch__pb2.BatchObjectsRequest.SerializeToString,
                response_deserializer=v1_dot_batch__pb2.BatchObjectsReply.FromString,
                _registered_method=True)
        self.BatchReferences = channel.unary_unary(
                '/weaviate.v1.Weaviate/BatchReferences',
                request_serializer=v1_dot_batch__pb2.BatchReferencesRequest.SerializeToString,
                response_deserializer=v1_dot_batch__pb2.BatchReferencesReply.FromString,
                _registered_method=True)
        self.BatchDelete = channel.unary_unary(
                '/weaviate.v1.Weaviate/BatchDelete',
                request_serializer=v1_dot_batch__delete__pb2.BatchDeleteRequest.SerializeToString,
                response_deserializer=v1_dot_batch__delete__pb2.BatchDeleteReply.FromString,
                _registered_method=True)
        self.TenantsGet = channel.unary_unary(
                '/weaviate.v1.Weaviate/TenantsGet',
                request_serializer=v1_dot_tenants__pb2.TenantsGetRequest.SerializeToString,
                response_deserializer=v1_dot_tenants__pb2.TenantsGetReply.FromString,
                _registered_method=True)
        self.Aggregate = channel.unary_unary(
                '/weaviate.v1.Weaviate/Aggregate',
                request_serializer=v1_dot_aggregate__pb2.AggregateRequest.SerializeToString,
                response_deserializer=v1_dot_aggregate__pb2.AggregateReply.FromString,
                _registered_method=True)
        self.BatchSend = channel.unary_unary(
                '/weaviate.v1.Weaviate/BatchSend',
                request_serializer=v1_dot_batch__pb2.BatchSendRequest.SerializeToString,
                response_deserializer=v1_dot_batch__pb2.BatchSendReply.FromString,
                _registered_method=True)
        self.BatchStream = channel.unary_stream(
                '/weaviate.v1.Weaviate/BatchStream',
                request_serializer=v1_dot_batch__pb2.BatchStreamRequest.SerializeToString,
                response_deserializer=v1_dot_batch__pb2.BatchStreamMessage.FromString,
                _registered_method=True)


class WeaviateServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Search(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BatchObjects(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BatchReferences(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BatchDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TenantsGet(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Aggregate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BatchSend(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BatchStream(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_WeaviateServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Search': grpc.unary_unary_rpc_method_handler(
                    servicer.Search,
                    request_deserializer=v1_dot_search__get__pb2.SearchRequest.FromString,
                    response_serializer=v1_dot_search__get__pb2.SearchReply.SerializeToString,
            ),
            'BatchObjects': grpc.unary_unary_rpc_method_handler(
                    servicer.BatchObjects,
                    request_deserializer=v1_dot_batch__pb2.BatchObjectsRequest.FromString,
                    response_serializer=v1_dot_batch__pb2.BatchObjectsReply.SerializeToString,
            ),
            'BatchReferences': grpc.unary_unary_rpc_method_handler(
                    servicer.BatchReferences,
                    request_deserializer=v1_dot_batch__pb2.BatchReferencesRequest.FromString,
                    response_serializer=v1_dot_batch__pb2.BatchReferencesReply.SerializeToString,
            ),
            'BatchDelete': grpc.unary_unary_rpc_method_handler(
                    servicer.BatchDelete,
                    request_deserializer=v1_dot_batch__delete__pb2.BatchDeleteRequest.FromString,
                    response_serializer=v1_dot_batch__delete__pb2.BatchDeleteReply.SerializeToString,
            ),
            'TenantsGet': grpc.unary_unary_rpc_method_handler(
                    servicer.TenantsGet,
                    request_deserializer=v1_dot_tenants__pb2.TenantsGetRequest.FromString,
                    response_serializer=v1_dot_tenants__pb2.TenantsGetReply.SerializeToString,
            ),
            'Aggregate': grpc.unary_unary_rpc_method_handler(
                    servicer.Aggregate,
                    request_deserializer=v1_dot_aggregate__pb2.AggregateRequest.FromString,
                    response_serializer=v1_dot_aggregate__pb2.AggregateReply.SerializeToString,
            ),
            'BatchSend': grpc.unary_unary_rpc_method_handler(
                    servicer.BatchSend,
                    request_deserializer=v1_dot_batch__pb2.BatchSendRequest.FromString,
                    response_serializer=v1_dot_batch__pb2.BatchSendReply.SerializeToString,
            ),
            'BatchStream': grpc.unary_stream_rpc_method_handler(
                    servicer.BatchStream,
                    request_deserializer=v1_dot_batch__pb2.BatchStreamRequest.FromString,
                    response_serializer=v1_dot_batch__pb2.BatchStreamMessage.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'weaviate.v1.Weaviate', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('weaviate.v1.Weaviate', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class Weaviate(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Search(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/weaviate.v1.Weaviate/Search',
            v1_dot_search__get__pb2.SearchRequest.SerializeToString,
            v1_dot_search__get__pb2.SearchReply.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def BatchObjects(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/weaviate.v1.Weaviate/BatchObjects',
            v1_dot_batch__pb2.BatchObjectsRequest.SerializeToString,
            v1_dot_batch__pb2.BatchObjectsReply.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def BatchReferences(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/weaviate.v1.Weaviate/BatchReferences',
            v1_dot_batch__pb2.BatchReferencesRequest.SerializeToString,
            v1_dot_batch__pb2.BatchReferencesReply.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def BatchDelete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/weaviate.v1.Weaviate/BatchDelete',
            v1_dot_batch__delete__pb2.BatchDeleteRequest.SerializeToString,
            v1_dot_batch__delete__pb2.BatchDeleteReply.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def TenantsGet(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/weaviate.v1.Weaviate/TenantsGet',
            v1_dot_tenants__pb2.TenantsGetRequest.SerializeToString,
            v1_dot_tenants__pb2.TenantsGetReply.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Aggregate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/weaviate.v1.Weaviate/Aggregate',
            v1_dot_aggregate__pb2.AggregateRequest.SerializeToString,
            v1_dot_aggregate__pb2.AggregateReply.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def BatchSend(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/weaviate.v1.Weaviate/BatchSend',
            v1_dot_batch__pb2.BatchSendRequest.SerializeToString,
            v1_dot_batch__pb2.BatchSendReply.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def BatchStream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/weaviate.v1.Weaviate/BatchStream',
            v1_dot_batch__pb2.BatchStreamRequest.SerializeToString,
            v1_dot_batch__pb2.BatchStreamMessage.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



from weaviate.proto.v1.v6300.v1 import base_pb2 as _base_pb2
from weaviate.proto.v1.v6300.v1 import base_search_pb2 as _base_search_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class AggregateRequest(_message.Message):
    __slots__ = ("collection", "tenant", "objects_count", "aggregations", "object_limit", "group_by", "limit", "filters", "hybrid", "near_vector", "near_object", "near_text", "near_image", "near_audio", "near_video", "near_depth", "near_thermal", "near_imu")
    class Aggregation(_message.Message):
        __slots__ = ("property", "int", "number", "text", "boolean", "date", "reference")
        class Integer(_message.Message):
            __slots__ = ("count", "type", "sum", "mean", "mode", "median", "maximum", "minimum")
            COUNT_FIELD_NUMBER: _ClassVar[int]
            TYPE_FIELD_NUMBER: _ClassVar[int]
            SUM_FIELD_NUMBER: _ClassVar[int]
            MEAN_FIELD_NUMBER: _ClassVar[int]
            MODE_FIELD_NUMBER: _ClassVar[int]
            MEDIAN_FIELD_NUMBER: _ClassVar[int]
            MAXIMUM_FIELD_NUMBER: _ClassVar[int]
            MINIMUM_FIELD_NUMBER: _ClassVar[int]
            count: bool
            type: bool
            sum: bool
            mean: bool
            mode: bool
            median: bool
            maximum: bool
            minimum: bool
            def __init__(self, count: bool = ..., type: bool = ..., sum: bool = ..., mean: bool = ..., mode: bool = ..., median: bool = ..., maximum: bool = ..., minimum: bool = ...) -> None: ...
        class Number(_message.Message):
            __slots__ = ("count", "type", "sum", "mean", "mode", "median", "maximum", "minimum")
            COUNT_FIELD_NUMBER: _ClassVar[int]
            TYPE_FIELD_NUMBER: _ClassVar[int]
            SUM_FIELD_NUMBER: _ClassVar[int]
            MEAN_FIELD_NUMBER: _ClassVar[int]
            MODE_FIELD_NUMBER: _ClassVar[int]
            MEDIAN_FIELD_NUMBER: _ClassVar[int]
            MAXIMUM_FIELD_NUMBER: _ClassVar[int]
            MINIMUM_FIELD_NUMBER: _ClassVar[int]
            count: bool
            type: bool
            sum: bool
            mean: bool
            mode: bool
            median: bool
            maximum: bool
            minimum: bool
            def __init__(self, count: bool = ..., type: bool = ..., sum: bool = ..., mean: bool = ..., mode: bool = ..., median: bool = ..., maximum: bool = ..., minimum: bool = ...) -> None: ...
        class Text(_message.Message):
            __slots__ = ("count", "type", "top_occurences", "top_occurences_limit")
            COUNT_FIELD_NUMBER: _ClassVar[int]
            TYPE_FIELD_NUMBER: _ClassVar[int]
            TOP_OCCURENCES_FIELD_NUMBER: _ClassVar[int]
            TOP_OCCURENCES_LIMIT_FIELD_NUMBER: _ClassVar[int]
            count: bool
            type: bool
            top_occurences: bool
            top_occurences_limit: int
            def __init__(self, count: bool = ..., type: bool = ..., top_occurences: bool = ..., top_occurences_limit: _Optional[int] = ...) -> None: ...
        class Boolean(_message.Message):
            __slots__ = ("count", "type", "total_true", "total_false", "percentage_true", "percentage_false")
            COUNT_FIELD_NUMBER: _ClassVar[int]
            TYPE_FIELD_NUMBER: _ClassVar[int]
            TOTAL_TRUE_FIELD_NUMBER: _ClassVar[int]
            TOTAL_FALSE_FIELD_NUMBER: _ClassVar[int]
            PERCENTAGE_TRUE_FIELD_NUMBER: _ClassVar[int]
            PERCENTAGE_FALSE_FIELD_NUMBER: _ClassVar[int]
            count: bool
            type: bool
            total_true: bool
            total_false: bool
            percentage_true: bool
            percentage_false: bool
            def __init__(self, count: bool = ..., type: bool = ..., total_true: bool = ..., total_false: bool = ..., percentage_true: bool = ..., percentage_false: bool = ...) -> None: ...
        class Date(_message.Message):
            __slots__ = ("count", "type", "median", "mode", "maximum", "minimum")
            COUNT_FIELD_NUMBER: _ClassVar[int]
            TYPE_FIELD_NUMBER: _ClassVar[int]
            MEDIAN_FIELD_NUMBER: _ClassVar[int]
            MODE_FIELD_NUMBER: _ClassVar[int]
            MAXIMUM_FIELD_NUMBER: _ClassVar[int]
            MINIMUM_FIELD_NUMBER: _ClassVar[int]
            count: bool
            type: bool
            median: bool
            mode: bool
            maximum: bool
            minimum: bool
            def __init__(self, count: bool = ..., type: bool = ..., median: bool = ..., mode: bool = ..., maximum: bool = ..., minimum: bool = ...) -> None: ...
        class Reference(_message.Message):
            __slots__ = ("type", "pointing_to")
            TYPE_FIELD_NUMBER: _ClassVar[int]
            POINTING_TO_FIELD_NUMBER: _ClassVar[int]
            type: bool
            pointing_to: bool
            def __init__(self, type: bool = ..., pointing_to: bool = ...) -> None: ...
        PROPERTY_FIELD_NUMBER: _ClassVar[int]
        INT_FIELD_NUMBER: _ClassVar[int]
        NUMBER_FIELD_NUMBER: _ClassVar[int]
        TEXT_FIELD_NUMBER: _ClassVar[int]
        BOOLEAN_FIELD_NUMBER: _ClassVar[int]
        DATE_FIELD_NUMBER: _ClassVar[int]
        REFERENCE_FIELD_NUMBER: _ClassVar[int]
        property: str
        int: AggregateRequest.Aggregation.Integer
        number: AggregateRequest.Aggregation.Number
        text: AggregateRequest.Aggregation.Text
        boolean: AggregateRequest.Aggregation.Boolean
        date: AggregateRequest.Aggregation.Date
        reference: AggregateRequest.Aggregation.Reference
        def __init__(self, property: _Optional[str] = ..., int: _Optional[_Union[AggregateRequest.Aggregation.Integer, _Mapping]] = ..., number: _Optional[_Union[AggregateRequest.Aggregation.Number, _Mapping]] = ..., text: _Optional[_Union[AggregateRequest.Aggregation.Text, _Mapping]] = ..., boolean: _Optional[_Union[AggregateRequest.Aggregation.Boolean, _Mapping]] = ..., date: _Optional[_Union[AggregateRequest.Aggregation.Date, _Mapping]] = ..., reference: _Optional[_Union[AggregateRequest.Aggregation.Reference, _Mapping]] = ...) -> None: ...
    class GroupBy(_message.Message):
        __slots__ = ("collection", "property")
        COLLECTION_FIELD_NUMBER: _ClassVar[int]
        PROPERTY_FIELD_NUMBER: _ClassVar[int]
        collection: str
        property: str
        def __init__(self, collection: _Optional[str] = ..., property: _Optional[str] = ...) -> None: ...
    COLLECTION_FIELD_NUMBER: _ClassVar[int]
    TENANT_FIELD_NUMBER: _ClassVar[int]
    OBJECTS_COUNT_FIELD_NUMBER: _ClassVar[int]
    AGGREGATIONS_FIELD_NUMBER: _ClassVar[int]
    OBJECT_LIMIT_FIELD_NUMBER: _ClassVar[int]
    GROUP_BY_FIELD_NUMBER: _ClassVar[int]
    LIMIT_FIELD_NUMBER: _ClassVar[int]
    FILTERS_FIELD_NUMBER: _ClassVar[int]
    HYBRID_FIELD_NUMBER: _ClassVar[int]
    NEAR_VECTOR_FIELD_NUMBER: _ClassVar[int]
    NEAR_OBJECT_FIELD_NUMBER: _ClassVar[int]
    NEAR_TEXT_FIELD_NUMBER: _ClassVar[int]
    NEAR_IMAGE_FIELD_NUMBER: _ClassVar[int]
    NEAR_AUDIO_FIELD_NUMBER: _ClassVar[int]
    NEAR_VIDEO_FIELD_NUMBER: _ClassVar[int]
    NEAR_DEPTH_FIELD_NUMBER: _ClassVar[int]
    NEAR_THERMAL_FIELD_NUMBER: _ClassVar[int]
    NEAR_IMU_FIELD_NUMBER: _ClassVar[int]
    collection: str
    tenant: str
    objects_count: bool
    aggregations: _containers.RepeatedCompositeFieldContainer[AggregateRequest.Aggregation]
    object_limit: int
    group_by: AggregateRequest.GroupBy
    limit: int
    filters: _base_pb2.Filters
    hybrid: _base_search_pb2.Hybrid
    near_vector: _base_search_pb2.NearVector
    near_object: _base_search_pb2.NearObject
    near_text: _base_search_pb2.NearTextSearch
    near_image: _base_search_pb2.NearImageSearch
    near_audio: _base_search_pb2.NearAudioSearch
    near_video: _base_search_pb2.NearVideoSearch
    near_depth: _base_search_pb2.NearDepthSearch
    near_thermal: _base_search_pb2.NearThermalSearch
    near_imu: _base_search_pb2.NearIMUSearch
    def __init__(self, collection: _Optional[str] = ..., tenant: _Optional[str] = ..., objects_count: bool = ..., aggregations: _Optional[_Iterable[_Union[AggregateRequest.Aggregation, _Mapping]]] = ..., object_limit: _Optional[int] = ..., group_by: _Optional[_Union[AggregateRequest.GroupBy, _Mapping]] = ..., limit: _Optional[int] = ..., filters: _Optional[_Union[_base_pb2.Filters, _Mapping]] = ..., hybrid: _Optional[_Union[_base_search_pb2.Hybrid, _Mapping]] = ..., near_vector: _Optional[_Union[_base_search_pb2.NearVector, _Mapping]] = ..., near_object: _Optional[_Union[_base_search_pb2.NearObject, _Mapping]] = ..., near_text: _Optional[_Union[_base_search_pb2.NearTextSearch, _Mapping]] = ..., near_image: _Optional[_Union[_base_search_pb2.NearImageSearch, _Mapping]] = ..., near_audio: _Optional[_Union[_base_search_pb2.NearAudioSearch, _Mapping]] = ..., near_video: _Optional[_Union[_base_search_pb2.NearVideoSearch, _Mapping]] = ..., near_depth: _Optional[_Union[_base_search_pb2.NearDepthSearch, _Mapping]] = ..., near_thermal: _Optional[_Union[_base_search_pb2.NearThermalSearch, _Mapping]] = ..., near_imu: _Optional[_Union[_base_search_pb2.NearIMUSearch, _Mapping]] = ...) -> None: ...

class AggregateReply(_message.Message):
    __slots__ = ("took", "single_result", "grouped_results")
    class Aggregations(_message.Message):
        __slots__ = ("aggregations",)
        class Aggregation(_message.Message):
            __slots__ = ("property", "int", "number", "text", "boolean", "date", "reference")
            class Integer(_message.Message):
                __slots__ = ("count", "type", "mean", "median", "mode", "maximum", "minimum", "sum")
                COUNT_FIELD_NUMBER: _ClassVar[int]
                TYPE_FIELD_NUMBER: _ClassVar[int]
                MEAN_FIELD_NUMBER: _ClassVar[int]
                MEDIAN_FIELD_NUMBER: _ClassVar[int]
                MODE_FIELD_NUMBER: _ClassVar[int]
                MAXIMUM_FIELD_NUMBER: _ClassVar[int]
                MINIMUM_FIELD_NUMBER: _ClassVar[int]
                SUM_FIELD_NUMBER: _ClassVar[int]
                count: int
                type: str
                mean: float
                median: float
                mode: int
                maximum: int
                minimum: int
                sum: int
                def __init__(self, count: _Optional[int] = ..., type: _Optional[str] = ..., mean: _Optional[float] = ..., median: _Optional[float] = ..., mode: _Optional[int] = ..., maximum: _Optional[int] = ..., minimum: _Optional[int] = ..., sum: _Optional[int] = ...) -> None: ...
            class Number(_message.Message):
                __slots__ = ("count", "type", "mean", "median", "mode", "maximum", "minimum", "sum")
                COUNT_FIELD_NUMBER: _ClassVar[int]
                TYPE_FIELD_NUMBER: _ClassVar[int]
                MEAN_FIELD_NUMBER: _ClassVar[int]
                MEDIAN_FIELD_NUMBER: _ClassVar[int]
                MODE_FIELD_NUMBER: _ClassVar[int]
                MAXIMUM_FIELD_NUMBER: _ClassVar[int]
                MINIMUM_FIELD_NUMBER: _ClassVar[int]
                SUM_FIELD_NUMBER: _ClassVar[int]
                count: int
                type: str
                mean: float
                median: float
                mode: float
                maximum: float
                minimum: float
                sum: float
                def __init__(self, count: _Optional[int] = ..., type: _Optional[str] = ..., mean: _Optional[float] = ..., median: _Optional[float] = ..., mode: _Optional[float] = ..., maximum: _Optional[float] = ..., minimum: _Optional[float] = ..., sum: _Optional[float] = ...) -> None: ...
            class Text(_message.Message):
                __slots__ = ("count", "type", "top_occurences")
                class TopOccurrences(_message.Message):
                    __slots__ = ("items",)
                    class TopOccurrence(_message.Message):
                        __slots__ = ("value", "occurs")
                        VALUE_FIELD_NUMBER: _ClassVar[int]
                        OCCURS_FIELD_NUMBER: _ClassVar[int]
                        value: str
                        occurs: int
                        def __init__(self, value: _Optional[str] = ..., occurs: _Optional[int] = ...) -> None: ...
                    ITEMS_FIELD_NUMBER: _ClassVar[int]
                    items: _containers.RepeatedCompositeFieldContainer[AggregateReply.Aggregations.Aggregation.Text.TopOccurrences.TopOccurrence]
                    def __init__(self, items: _Optional[_Iterable[_Union[AggregateReply.Aggregations.Aggregation.Text.TopOccurrences.TopOccurrence, _Mapping]]] = ...) -> None: ...
                COUNT_FIELD_NUMBER: _ClassVar[int]
                TYPE_FIELD_NUMBER: _ClassVar[int]
                TOP_OCCURENCES_FIELD_NUMBER: _ClassVar[int]
                count: int
                type: str
                top_occurences: AggregateReply.Aggregations.Aggregation.Text.TopOccurrences
                def __init__(self, count: _Optional[int] = ..., type: _Optional[str] = ..., top_occurences: _Optional[_Union[AggregateReply.Aggregations.Aggregation.Text.TopOccurrences, _Mapping]] = ...) -> None: ...
            class Boolean(_message.Message):
                __slots__ = ("count", "type", "total_true", "total_false", "percentage_true", "percentage_false")
                COUNT_FIELD_NUMBER: _ClassVar[int]
                TYPE_FIELD_NUMBER: _ClassVar[int]
                TOTAL_TRUE_FIELD_NUMBER: _ClassVar[int]
                TOTAL_FALSE_FIELD_NUMBER: _ClassVar[int]
                PERCENTAGE_TRUE_FIELD_NUMBER: _ClassVar[int]
                PERCENTAGE_FALSE_FIELD_NUMBER: _ClassVar[int]
                count: int
                type: str
                total_true: int
                total_false: int
                percentage_true: float
                percentage_false: float
                def __init__(self, count: _Optional[int] = ..., type: _Optional[str] = ..., total_true: _Optional[int] = ..., total_false: _Optional[int] = ..., percentage_true: _Optional[float] = ..., percentage_false: _Optional[float] = ...) -> None: ...
            class Date(_message.Message):
                __slots__ = ("count", "type", "median", "mode", "maximum", "minimum")
                COUNT_FIELD_NUMBER: _ClassVar[int]
                TYPE_FIELD_NUMBER: _ClassVar[int]
                MEDIAN_FIELD_NUMBER: _ClassVar[int]
                MODE_FIELD_NUMBER: _ClassVar[int]
                MAXIMUM_FIELD_NUMBER: _ClassVar[int]
                MINIMUM_FIELD_NUMBER: _ClassVar[int]
                count: int
                type: str
                median: str
                mode: str
                maximum: str
                minimum: str
                def __init__(self, count: _Optional[int] = ..., type: _Optional[str] = ..., median: _Optional[str] = ..., mode: _Optional[str] = ..., maximum: _Optional[str] = ..., minimum: _Optional[str] = ...) -> None: ...
            class Reference(_message.Message):
                __slots__ = ("type", "pointing_to")
                TYPE_FIELD_NUMBER: _ClassVar[int]
                POINTING_TO_FIELD_NUMBER: _ClassVar[int]
                type: str
                pointing_to: _containers.RepeatedScalarFieldContainer[str]
                def __init__(self, type: _Optional[str] = ..., pointing_to: _Optional[_Iterable[str]] = ...) -> None: ...
            PROPERTY_FIELD_NUMBER: _ClassVar[int]
            INT_FIELD_NUMBER: _ClassVar[int]
            NUMBER_FIELD_NUMBER: _ClassVar[int]
            TEXT_FIELD_NUMBER: _ClassVar[int]
            BOOLEAN_FIELD_NUMBER: _ClassVar[int]
            DATE_FIELD_NUMBER: _ClassVar[int]
            REFERENCE_FIELD_NUMBER: _ClassVar[int]
            property: str
            int: AggregateReply.Aggregations.Aggregation.Integer
            number: AggregateReply.Aggregations.Aggregation.Number
            text: AggregateReply.Aggregations.Aggregation.Text
            boolean: AggregateReply.Aggregations.Aggregation.Boolean
            date: AggregateReply.Aggregations.Aggregation.Date
            reference: AggregateReply.Aggregations.Aggregation.Reference
            def __init__(self, property: _Optional[str] = ..., int: _Optional[_Union[AggregateReply.Aggregations.Aggregation.Integer, _Mapping]] = ..., number: _Optional[_Union[AggregateReply.Aggregations.Aggregation.Number, _Mapping]] = ..., text: _Optional[_Union[AggregateReply.Aggregations.Aggregation.Text, _Mapping]] = ..., boolean: _Optional[_Union[AggregateReply.Aggregations.Aggregation.Boolean, _Mapping]] = ..., date: _Optional[_Union[AggregateReply.Aggregations.Aggregation.Date, _Mapping]] = ..., reference: _Optional[_Union[AggregateReply.Aggregations.Aggregation.Reference, _Mapping]] = ...) -> None: ...
        AGGREGATIONS_FIELD_NUMBER: _ClassVar[int]
        aggregations: _containers.RepeatedCompositeFieldContainer[AggregateReply.Aggregations.Aggregation]
        def __init__(self, aggregations: _Optional[_Iterable[_Union[AggregateReply.Aggregations.Aggregation, _Mapping]]] = ...) -> None: ...
    class Single(_message.Message):
        __slots__ = ("objects_count", "aggregations")
        OBJECTS_COUNT_FIELD_NUMBER: _ClassVar[int]
        AGGREGATIONS_FIELD_NUMBER: _ClassVar[int]
        objects_count: int
        aggregations: AggregateReply.Aggregations
        def __init__(self, objects_count: _Optional[int] = ..., aggregations: _Optional[_Union[AggregateReply.Aggregations, _Mapping]] = ...) -> None: ...
    class Group(_message.Message):
        __slots__ = ("objects_count", "aggregations", "grouped_by")
        class GroupedBy(_message.Message):
            __slots__ = ("path", "text", "int", "boolean", "number", "texts", "ints", "booleans", "numbers", "geo")
            PATH_FIELD_NUMBER: _ClassVar[int]
            TEXT_FIELD_NUMBER: _ClassVar[int]
            INT_FIELD_NUMBER: _ClassVar[int]
            BOOLEAN_FIELD_NUMBER: _ClassVar[int]
            NUMBER_FIELD_NUMBER: _ClassVar[int]
            TEXTS_FIELD_NUMBER: _ClassVar[int]
            INTS_FIELD_NUMBER: _ClassVar[int]
            BOOLEANS_FIELD_NUMBER: _ClassVar[int]
            NUMBERS_FIELD_NUMBER: _ClassVar[int]
            GEO_FIELD_NUMBER: _ClassVar[int]
            path: _containers.RepeatedScalarFieldContainer[str]
            text: str
            int: int
            boolean: bool
            number: float
            texts: _base_pb2.TextArray
            ints: _base_pb2.IntArray
            booleans: _base_pb2.BooleanArray
            numbers: _base_pb2.NumberArray
            geo: _base_pb2.GeoCoordinatesFilter
            def __init__(self, path: _Optional[_Iterable[str]] = ..., text: _Optional[str] = ..., int: _Optional[int] = ..., boolean: bool = ..., number: _Optional[float] = ..., texts: _Optional[_Union[_base_pb2.TextArray, _Mapping]] = ..., ints: _Optional[_Union[_base_pb2.IntArray, _Mapping]] = ..., booleans: _Optional[_Union[_base_pb2.BooleanArray, _Mapping]] = ..., numbers: _Optional[_Union[_base_pb2.NumberArray, _Mapping]] = ..., geo: _Optional[_Union[_base_pb2.GeoCoordinatesFilter, _Mapping]] = ...) -> None: ...
        OBJECTS_COUNT_FIELD_NUMBER: _ClassVar[int]
        AGGREGATIONS_FIELD_NUMBER: _ClassVar[int]
        GROUPED_BY_FIELD_NUMBER: _ClassVar[int]
        objects_count: int
        aggregations: AggregateReply.Aggregations
        grouped_by: AggregateReply.Group.GroupedBy
        def __init__(self, objects_count: _Optional[int] = ..., aggregations: _Optional[_Union[AggregateReply.Aggregations, _Mapping]] = ..., grouped_by: _Optional[_Union[AggregateReply.Group.GroupedBy, _Mapping]] = ...) -> None: ...
    class Grouped(_message.Message):
        __slots__ = ("groups",)
        GROUPS_FIELD_NUMBER: _ClassVar[int]
        groups: _containers.RepeatedCompositeFieldContainer[AggregateReply.Group]
        def __init__(self, groups: _Optional[_Iterable[_Union[AggregateReply.Group, _Mapping]]] = ...) -> None: ...
    TOOK_FIELD_NUMBER: _ClassVar[int]
    SINGLE_RESULT_FIELD_NUMBER: _ClassVar[int]
    GROUPED_RESULTS_FIELD_NUMBER: _ClassVar[int]
    took: float
    single_result: AggregateReply.Single
    grouped_results: AggregateReply.Grouped
    def __init__(self, took: _Optional[float] = ..., single_result: _Optional[_Union[AggregateReply.Single, _Mapping]] = ..., grouped_results: _Optional[_Union[AggregateReply.Grouped, _Mapping]] = ...) -> None: ...



from .generate import _NearObjectGenerate, _NearObjectGenerateAsync
from .query import _NearObjectQuery, _NearObjectQueryAsync

__all__ = [
    "_NearObjectGenerate",
    "_NearObjectGenerateAsync",
    "_NearObjectQuery",
    "_NearObjectQueryAsync",
]



import uuid
from typing import Any

import numpy as np
import pandas as pd
import polars as pl
import pytest

from integration.conftest import CollectionFactory
from weaviate.collections.classes.config import (
    Configure,
    DataType,
    Property,
)
from weaviate.collections.classes.grpc import (
    GroupBy,
    MetadataQuery,
)
from weaviate.exceptions import WeaviateInvalidInputError

UUID1 = uuid.UUID("806827e0-2b31-43ca-9269-24fa95a221f9")
UUID2 = uuid.UUID("8ad0d33c-8db1-4437-87f3-72161ca2a51a")
UUID3 = uuid.UUID("83d99755-9deb-4b16-8431-d1dff4ab0a75")


def test_near_vector(collection_factory: CollectionFactory) -> None:
    collection = collection_factory(
        properties=[Property(name="Name", data_type=DataType.TEXT)],
        vectorizer_config=Configure.Vectorizer.text2vec_contextionary(
            vectorize_collection_name=False
        ),
    )
    uuid_banana = collection.data.insert({"Name": "Banana"})
    collection.data.insert({"Name": "Fruit"})
    collection.data.insert({"Name": "car"})
    collection.data.insert({"Name": "Mountain"})

    banana = collection.query.fetch_object_by_id(uuid_banana, include_vector=True)

    full_objects = collection.query.near_vector(
        banana.vector["default"], return_metadata=MetadataQuery(distance=True, certainty=True)
    ).objects
    assert len(full_objects) == 4

    objects_distance = collection.query.near_vector(
        banana.vector["default"], distance=full_objects[2].metadata.distance
    ).objects
    assert len(objects_distance) == 3

    objects_distance = collection.query.near_vector(
        banana.vector["default"], certainty=full_objects[2].metadata.certainty
    ).objects
    assert len(objects_distance) == 3


def test_near_vector_limit(collection_factory: CollectionFactory) -> None:
    collection = collection_factory(
        properties=[Property(name="Name", data_type=DataType.TEXT)],
        vectorizer_config=Configure.Vectorizer.text2vec_contextionary(
            vectorize_collection_name=False
        ),
    )
    uuid_banana = collection.data.insert({"Name": "Banana"})
    collection.data.insert({"Name": "Fruit"})
    collection.data.insert({"Name": "car"})
    collection.data.insert({"Name": "Mountain"})

    banana = collection.query.fetch_object_by_id(uuid_banana, include_vector=True)

    objs = collection.query.near_vector(banana.vector["default"], limit=2).objects
    assert len(objs) == 2


def test_near_vector_offset(collection_factory: CollectionFactory) -> None:
    collection = collection_factory(
        properties=[Property(name="Name", data_type=DataType.TEXT)],
        vectorizer_config=Configure.Vectorizer.text2vec_contextionary(
            vectorize_collection_name=False
        ),
    )
    uuid_banana = collection.data.insert({"Name": "Banana"})
    uuid_fruit = collection.data.insert({"Name": "Fruit"})
    collection.data.insert({"Name": "car"})
    collection.data.insert({"Name": "Mountain"})

    banana = collection.query.fetch_object_by_id(uuid_banana, include_vector=True)

    objs = collection.query.near_vector(banana.vector["default"], offset=1).objects
    assert len(objs) == 3
    assert objs[0].uuid == uuid_fruit


def test_near_vector_group_by_argument(collection_factory: CollectionFactory) -> None:
    collection = collection_factory(
        properties=[
            Property(name="Name", data_type=DataType.TEXT),
            Property(name="Count", data_type=DataType.INT),
        ],
        vectorizer_config=Configure.Vectorizer.text2vec_contextionary(
            vectorize_collection_name=False
        ),
    )
    uuid_banana1 = collection.data.insert({"Name": "Banana", "Count": 51})
    collection.data.insert({"Name": "Banana", "Count": 72})
    collection.data.insert({"Name": "car", "Count": 12})
    collection.data.insert({"Name": "Mountain", "Count": 1})

    banana1 = collection.query.fetch_object_by_id(uuid_banana1, include_vector=True)

    ret = collection.query.near_vector(
        banana1.vector["default"],
        group_by=GroupBy(
            prop="name",
            number_of_groups=4,
            objects_per_group=10,
        ),
        return_metadata=MetadataQuery(distance=True, certainty=True),
    )

    assert len(ret.objects) == 4
    assert ret.objects[0].belongs_to_group == "Banana"
    assert ret.objects[1].belongs_to_group == "Banana"
    assert ret.objects[2].belongs_to_group == "car"
    assert ret.objects[3].belongs_to_group == "Mountain"


@pytest.mark.parametrize(
    "near_vector", [[1, 0], [1.0, 0.0], np.array([1, 0]), pl.Series([1, 0]), pd.Series([1, 0])]
)
def test_near_vector_with_other_input(
    collection_factory: CollectionFactory, near_vector: Any
) -> None:
    collection = collection_factory(vectorizer_config=Configure.Vectorizer.none())

    uuid1 = collection.data.insert({}, vector=[1, 0])
    collection.data.insert({}, vector=[0, 1])

    ret = collection.query.near_vector(
        near_vector,
        distance=0.1,
    )
    assert len(ret.objects) == 1
    assert ret.objects[0].uuid == uuid1


@pytest.mark.parametrize(
    "near_vector",
    [
        {"first": [1, 0], "second": [1, 0, 0]},
        {"first": np.array([1, 0]), "second": [1, 0, 0]},
        {"first": pl.Series([1, 0]), "second": [1, 0, 0]},
        {"first": pd.Series([1, 0]), "second": [1, 0, 0]},
        {"first": [1.0, 0.0], "second": [1.0, 0.0, 0.0]},
    ],
)
def test_near_vector_with_named_vector_other_input(
    collection_factory: CollectionFactory, near_vector: Any
) -> None:
    dummy = collection_factory("dummy")
    if dummy._connection._weaviate_version.is_lower_than(1, 26, 0):
        pytest.skip("Named vectors are supported in versions higher than 1.26.0")

    collection = collection_factory(
        vectorizer_config=[
            Configure.NamedVectors.none("first"),
            Configure.NamedVectors.none("second"),
        ]
    )

    uuid1 = collection.data.insert({}, vector={"first": [1, 0], "second": [1, 0, 0]})
    collection.data.insert({}, vector={"first": [0, 1], "second": [0, 0, 1]})

    ret = collection.query.near_vector(near_vector, distance=0.1, target_vector=["first", "second"])
    assert len(ret.objects) == 1
    assert ret.objects[0].uuid == uuid1


def test_near_vector_with_extra_vectors(collection_factory: CollectionFactory):
    dummy = collection_factory("dummy")
    if dummy._connection._weaviate_version.is_lower_than(1, 26, 0):
        pytest.skip("Named vectors are supported in versions higher than 1.26.0")

    collection = collection_factory(
        vectorizer_config=[
            Configure.NamedVectors.none("first"),
            Configure.NamedVectors.none("second"),
        ]
    )

    collection.data.insert({}, vector={"first": [1, 0], "second": [1, 0, 0]})
    with pytest.raises(WeaviateInvalidInputError):
        collection.query.near_vector(
            {"first": [1, 0], "second": [1, 0, 0]}, target_vector=["second"]
        )



import pytest

import weaviate.classes as wvc


@pytest.mark.parametrize("expires_in,warning", [(-1, True), (5, False)])
def test_bearer_validation(
    recwarn: pytest.WarningsRecorder, expires_in: int, warning: bool
) -> None:
    wvc.init.Auth.bearer_token(
        access_token="Doesn't matter",
        refresh_token="Doesn't matter",
        expires_in=expires_in,
    )

    if warning:
        assert len(recwarn) == 1
        w = recwarn.pop()
        assert issubclass(w.category, UserWarning)
        assert str(w.message).startswith("Auth003")
    else:
        assert len(recwarn) == 0



from typing import Generic, Optional

from weaviate.collections.classes.config import ConsistencyLevel
from weaviate.connect.v4 import ConnectionType
from weaviate.util import _capitalize_first_letter


class _CollectionBase(Generic[ConnectionType]):
    def __init__(
        self,
        connection: ConnectionType,
        name: str,
        validate_arguments: bool,
        consistency_level: Optional[ConsistencyLevel] = None,
        tenant: Optional[str] = None,
    ) -> None:
        self._connection = connection
        self.name = _capitalize_first_letter(name)
        self._validate_arguments = validate_arguments

        self.__tenant = tenant
        self.__consistency_level = consistency_level

    @property
    def tenant(self) -> Optional[str]:
        """The tenant of this collection object."""
        return self.__tenant

    @property
    def consistency_level(self) -> Optional[ConsistencyLevel]:
        """The consistency level of this collection object."""
        return self.__consistency_level



import time
import pytest
from integration.conftest import ClientFactory, CollectionFactory

import weaviate

PORTS = (8087, 50058)

pytestmark = pytest.mark.xdist_group(name="replicate")


def replicate_collection(collection_factory: CollectionFactory):
    dummy = collection_factory("dummy", ports=PORTS)
    if dummy._connection._weaviate_version.is_lower_than(1, 32, 0):
        pytest.skip("Replication operations require Weaviate 1.32.0 or higher")

    collection = collection_factory(
        properties=[
            weaviate.classes.config.Property(
                name="name", data_type=weaviate.classes.config.DataType.TEXT
            )
        ],
        ports=PORTS,
        replication_config=weaviate.classes.config.Configure.replication(factor=1),
    )
    time.sleep(1)  # wait of eventual consistency of collection creation
    collection.data.insert_many(
        [
            {"name": name}
            for name in [
                "Alice",
                "Bob",
                "Charlie",
                "David",
                "Eve",
                "Frank",
                "Grace",
                "Heidi",
                "Ivan",
                "Judy",
                "Karl",
                "Leo",
                "Mallory",
                "Nina",
                "Oscar",
                "Peggy",
                "Quentin",
                "Rupert",
                "Sybil",
                "Trent",
                "Uma",
                "Victor",
                "Walter",
                "Xena",
                "Yara",
                "Zane",
            ]
        ]
    )
    return collection


@pytest.fixture
def replicate_client(client_factory: ClientFactory):
    return client_factory(
        ports=PORTS,
    )


def cleanup_ops(replicate_client: weaviate.WeaviateClient):
    """Cleanup any existing replication operations before running tests."""
    replicate_client.cluster.replications.delete_all()
    while len(replicate_client.cluster.replications.list_all()) > 0:
        time.sleep(0.1)


def test_replicate_and_get(
    replicate_client: weaviate.WeaviateClient, collection_factory: CollectionFactory
):
    collection = replicate_collection(collection_factory)

    nodes = replicate_client.cluster.nodes(collection=collection.name, output="verbose")
    src_node = nodes[0].name
    tgt_node = nodes[1].name
    shard = nodes[0].shards[0].name

    cleanup_ops(replicate_client)
    op_id = replicate_client.cluster.replicate(
        collection=collection.name,
        shard=shard,
        source_node=src_node,
        target_node=tgt_node,
    )

    op1 = replicate_client.cluster.replications.get(uuid=op_id)

    assert op1 is not None
    assert op1.collection == collection.name
    assert op1.shard == shard
    assert op1.source_node == src_node
    assert op1.target_node == tgt_node
    assert op1.transfer_type == weaviate.classes.replication.ReplicationType.COPY
    assert op1.status is not None
    assert op1.status_history is None

    op2 = replicate_client.cluster.replications.get(uuid=op_id, include_history=True)

    assert op2 is not None
    assert op2.collection == collection.name
    assert op2.shard == shard
    assert op2.source_node == src_node
    assert op2.target_node == tgt_node
    assert op2.transfer_type == weaviate.classes.replication.ReplicationType.COPY
    assert op2.status is not None
    assert op2.status_history is not None


def test_replicate_and_cancel(
    replicate_client: weaviate.WeaviateClient, collection_factory: CollectionFactory
):
    collection = replicate_collection(collection_factory)

    nodes = replicate_client.cluster.nodes(collection=collection.name, output="verbose")
    src_node = nodes[0].name
    tgt_node = nodes[1].name
    shard = nodes[0].shards[0].name

    cleanup_ops(replicate_client)
    op_id = replicate_client.cluster.replicate(
        collection=collection.name,
        shard=shard,
        source_node=src_node,
        target_node=tgt_node,
    )

    replicate_client.cluster.replications.cancel(uuid=op_id)

    start = time.time()
    while (
        not replicate_client.cluster.replications.get(uuid=op_id).status.state  # type: ignore
        == weaviate.outputs.replication.ReplicateOperationState.CANCELLED
    ):
        time.sleep(0.1)
        if time.time() - start > 10:
            raise TimeoutError("Timed out waiting for replication operation to be cancelled")


def test_replicate_and_delete(
    replicate_client: weaviate.WeaviateClient, collection_factory: CollectionFactory
):
    collection = replicate_collection(collection_factory)

    nodes = replicate_client.cluster.nodes(collection=collection.name, output="verbose")
    src_node = nodes[0].name
    tgt_node = nodes[1].name
    shard = nodes[0].shards[0].name

    cleanup_ops(replicate_client)
    op_id = replicate_client.cluster.replicate(
        collection=collection.name,
        shard=shard,
        source_node=src_node,
        target_node=tgt_node,
    )

    replicate_client.cluster.replications.delete(uuid=op_id)

    start = time.time()
    while replicate_client.cluster.replications.get(uuid=op_id) is not None:
        time.sleep(0.1)
        if time.time() - start > 10:
            raise TimeoutError("Timed out waiting for replication operation to be deleted")


def test_replicate_and_query(
    replicate_client: weaviate.WeaviateClient, collection_factory: CollectionFactory
):
    collection = replicate_collection(collection_factory)

    nodes = replicate_client.cluster.nodes(collection=collection.name, output="verbose")
    src_node = nodes[0].name
    tgt_node = nodes[1].name
    shard = nodes[0].shards[0].name

    cleanup_ops(replicate_client)
    replicate_client.cluster.replicate(
        collection=collection.name,
        shard=shard,
        source_node=src_node,
        target_node=tgt_node,
    )

    ops = replicate_client.cluster.replications.query(
        collection=collection.name,
        shard=shard,
        target_node=tgt_node,
    )
    assert len(ops) == 1

    ops = replicate_client.cluster.replications.query(
        collection=collection.name,
        shard=shard,
        target_node=src_node,
    )
    assert len(ops) == 1

    ops = replicate_client.cluster.replications.list_all()
    assert len(ops) == 1


def test_query_sharding_state(
    replicate_client: weaviate.WeaviateClient,
    collection_factory: CollectionFactory,
):
    collection = replicate_collection(collection_factory)

    nodes = replicate_client.cluster.nodes(collection=collection.name, output="verbose")
    shard = nodes[0].shards[0].name

    sharding_state = replicate_client.cluster.query_sharding_state(collection=collection.name)
    assert sharding_state is not None
    assert shard in [s.name for s in sharding_state.shards]

    sharding_state = replicate_client.cluster.query_sharding_state(
        collection=collection.name, shard=shard
    )
    assert sharding_state is not None
    assert shard in [s.name for s in sharding_state.shards]

    assert (
        replicate_client.cluster.query_sharding_state(collection="non_existent_collection") is None
    )
    assert (
        replicate_client.cluster.query_sharding_state(
            collection=collection.name, shard="non_existent_shard"
        )
        is None
    )



from .generate import _BM25Generate, _BM25GenerateAsync
from .query import _BM25Query, _BM25QueryAsync

__all__ = [
    "_BM25GenerateAsync",
    "_BM25QueryAsync",
    "_BM25Generate",
    "_BM25Query",
]



from typing import Dict, List, Literal, Optional, Union, overload

from typing_extensions import deprecated

from weaviate.collections.classes.config import (
    CollectionConfig,
    CollectionConfigSimple,
    Property,
    ReferenceProperty,
    ShardStatus,
    ShardTypes,
    _GenerativeProvider,
    _InvertedIndexConfigUpdate,
    _MultiTenancyConfigUpdate,
    _NamedVectorConfigCreate,
    _NamedVectorConfigUpdate,
    _ReferencePropertyMultiTarget,
    _ReplicationConfigUpdate,
    _RerankerProvider,
    _VectorConfigCreate,
    _VectorConfigUpdate,
    _VectorIndexConfigFlatUpdate,
    _VectorIndexConfigHNSWUpdate,
)
from weaviate.collections.classes.config_vector_index import _VectorIndexConfigDynamicUpdate
from weaviate.connect.v4 import ConnectionSync

from .executor import _ConfigCollectionExecutor

class _ConfigCollection(_ConfigCollectionExecutor[ConnectionSync]):
    @overload
    def get(self, simple: Literal[False] = False) -> CollectionConfig: ...
    @overload
    def get(self, simple: Literal[True]) -> CollectionConfigSimple: ...
    @overload
    def get(self, simple: bool = False) -> Union[CollectionConfig, CollectionConfigSimple]: ...
    def update(
        self,
        *,
        description: Optional[str] = None,
        property_descriptions: Optional[Dict[str, str]] = None,
        inverted_index_config: Optional[_InvertedIndexConfigUpdate] = None,
        multi_tenancy_config: Optional[_MultiTenancyConfigUpdate] = None,
        replication_config: Optional[_ReplicationConfigUpdate] = None,
        vector_index_config: Optional[
            Union[_VectorIndexConfigHNSWUpdate, _VectorIndexConfigFlatUpdate]
        ] = None,
        vectorizer_config: Optional[
            Union[
                _VectorIndexConfigHNSWUpdate,
                _VectorIndexConfigFlatUpdate,
                _VectorIndexConfigDynamicUpdate,
                List[_NamedVectorConfigUpdate],
            ]
        ] = None,
        vector_config: Optional[Union[_VectorConfigUpdate, List[_VectorConfigUpdate]]] = None,
        generative_config: Optional[_GenerativeProvider] = None,
        reranker_config: Optional[_RerankerProvider] = None,
    ) -> None: ...
    def get_shards(self) -> List[ShardStatus]: ...
    def update_shards(
        self,
        status: Literal["READY", "READONLY"],
        shard_names: Optional[Union[str, List[str]]] = None,
    ) -> Dict[str, ShardTypes]: ...
    def add_property(self, prop: Property) -> None: ...
    def add_reference(
        self, ref: Union[ReferenceProperty, _ReferencePropertyMultiTarget]
    ) -> None: ...
    @overload
    @deprecated(
        "Using `Configure.NamedVectors` in `vector_config` is deprecated. Instead, use `Configure.Vectors` or `Configure.MultiVectors`."
    )
    def add_vector(
        self, *, vector_config: Union[_NamedVectorConfigCreate, List[_NamedVectorConfigCreate]]
    ) -> None: ...
    @overload
    def add_vector(
        self, *, vector_config: Union[_VectorConfigCreate, List[_VectorConfigCreate]]
    ) -> None: ...



from dataclasses import dataclass
from typing import Generic, List, Literal, Optional, TypeVar, cast

from weaviate.cluster.types import Node as NodeREST
from weaviate.cluster.types import Shard as ShardREST


@dataclass
class Shard:
    """The properties of a single shard of a collection."""

    collection: str
    name: str
    node: str
    object_count: int
    vector_indexing_status: Literal["READONLY", "INDEXING", "READY", "LAZY_LOADING"]
    vector_queue_length: int
    compressed: bool
    loaded: Optional[bool]  # not present in <1.24.x


@dataclass
class Stats:
    """The statistics of a collection."""

    object_count: int
    shard_count: int


Shards = List[Shard]
Sh = TypeVar("Sh")
St = TypeVar("St")


@dataclass
class Node(Generic[Sh, St]):
    """The properties of a single node in the cluster."""

    git_hash: str
    name: str
    shards: Sh
    stats: St
    status: str
    version: str


NodeVerbose = Node[Shards, Stats]
NodeMinimal = Node[None, None]


class _ConvertFromREST:
    @staticmethod
    def nodes_verbose(nodes: List[NodeREST]) -> List[NodeVerbose]:
        return [
            Node(
                git_hash=node.get("gitHash", "None"),
                name=node["name"],
                shards=(
                    [
                        Shard(
                            collection=shard["class"],
                            name=shard["name"],
                            node=node["name"],
                            object_count=shard["objectCount"],
                            vector_indexing_status=shard["vectorIndexingStatus"],
                            vector_queue_length=shard["vectorQueueLength"],
                            compressed=shard["compressed"],
                            loaded=shard.get("loaded"),
                        )
                        for shard in cast(List[ShardREST], node["shards"])
                    ]
                    if "shards" in node and node["shards"] is not None
                    else []
                ),
                stats=(
                    Stats(
                        object_count=node["stats"]["objectCount"],
                        shard_count=node["stats"]["shardCount"],
                    )
                    if "stats" in node
                    else Stats(
                        object_count=0,
                        shard_count=0,
                    )
                ),
                status=node["status"],
                version=node.get("version", ""),
            )
            for node in nodes
        ]

    @staticmethod
    def nodes_minimal(nodes: List[NodeREST]) -> List[NodeMinimal]:
        return [
            Node(
                git_hash=node.get("gitHash", "None"),
                name=node["name"],
                shards=None,
                stats=None,
                status=node["status"],
                version=node.get("version", ""),
            )
            for node in nodes
        ]



from weaviate.aliases.executor import _AliasExecutor
from weaviate.connect import executor
from weaviate.connect.v4 import ConnectionAsync


@executor.wrap("async")
class _AliasAsync(_AliasExecutor[ConnectionAsync]):
    pass



from typing import Dict, Generic, List, Optional, cast

from httpx import Response

from weaviate.aliases.alias import AliasReturn, _WeaviateAlias
from weaviate.connect import executor
from weaviate.connect.v4 import Connection, ConnectionType, _ExpectedStatusCodes
from weaviate.util import _decode_json_response_dict


class _AliasExecutor(Generic[ConnectionType]):
    def __init__(self, connection: Connection):
        self._connection = connection

    def list_all(
        self, *, collection: Optional[str] = None
    ) -> executor.Result[Dict[str, AliasReturn]]:
        """Get the alias for a given alias name."""
        self._connection._weaviate_version.check_is_at_least_1_32_0("alias")

        error_msg = "list all aliases"
        if collection is not None:
            error_msg += f" for collection {collection}"

        def resp(res: Response) -> Dict[str, AliasReturn]:
            response_typed = _decode_json_response_dict(res, "list all aliases")
            assert response_typed is not None
            aliases = response_typed.get("aliases")
            assert aliases is not None, "Expected 'aliases' in response"
            return {
                alias["alias"]: AliasReturn(alias=alias["alias"], collection=alias["class"])
                for alias in cast(List[_WeaviateAlias], aliases)
            }

        return executor.execute(
            response_callback=resp,
            method=self._connection.get,
            path="/aliases",
            error_msg=error_msg,
            params={"class": collection} if collection else None,
            status_codes=_ExpectedStatusCodes(
                ok_in=[200],
                error=error_msg,
            ),
        )

    def get(self, *, alias_name: str) -> executor.Result[Optional[AliasReturn]]:
        """Get the given alias."""
        self._connection._weaviate_version.check_is_at_least_1_32_0("alias")

        def resp(res: Response) -> Optional[AliasReturn]:
            if res.status_code == 404:
                return None
            response_typed = _decode_json_response_dict(res, "get alias")
            assert response_typed is not None

            return AliasReturn(alias=response_typed["alias"], collection=response_typed["class"])

        return executor.execute(
            response_callback=resp,
            method=self._connection.get,
            path=f"/aliases/{alias_name}",
            error_msg=f"Could not get alias {alias_name}",
            status_codes=_ExpectedStatusCodes(
                ok_in=[200, 404],
                error="get alias",
            ),
        )

    def create(self, *, alias_name: str, target_collection: str) -> executor.Result[None]:
        """Create an alias for a given collection."""
        self._connection._weaviate_version.check_is_at_least_1_32_0("alias")

        def resp(res: Response) -> None:
            return None

        return executor.execute(
            response_callback=resp,
            method=self._connection.post,
            path="/aliases",
            error_msg=f"Could not create alias {alias_name} for collection {target_collection}",
            weaviate_object={"class": target_collection, "alias": alias_name},
            status_codes=_ExpectedStatusCodes(
                ok_in=[200],
                error="create aliases",
            ),
        )

    def delete(self, *, alias_name: str) -> executor.Result[bool]:
        """Create an alias."""
        self._connection._weaviate_version.check_is_at_least_1_32_0("alias")

        def resp(res: Response) -> bool:
            return res.status_code == 204

        return executor.execute(
            response_callback=resp,
            method=self._connection.delete,
            path=f"/aliases/{alias_name}",
            error_msg=f"Could not delete alias {alias_name}",
            status_codes=_ExpectedStatusCodes(
                ok_in=[204, 404],
                error="delete aliases",
            ),
        )

    def update(self, *, alias_name: str, new_target_collection: str) -> executor.Result[bool]:
        """Replace an alias."""
        self._connection._weaviate_version.check_is_at_least_1_32_0("alias")

        def resp(res: Response) -> bool:
            return res.status_code == 200

        return executor.execute(
            response_callback=resp,
            method=self._connection.put,
            path=f"/aliases/{alias_name}",
            weaviate_object={"class": new_target_collection},
            error_msg=f"Could not update alias {alias_name} to point to collection {new_target_collection}",
            status_codes=_ExpectedStatusCodes(
                ok_in=[200, 404],
                error="update aliases",
            ),
        )

    def exists(self, *, alias_name: str) -> executor.Result[bool]:
        """Use this method to check if an alias exists in the Weaviate instance.

        Args:
            name: The name of the alias to check.

        Returns:
            `True` if the alias exists, `False` otherwise.
        """
        self._connection._weaviate_version.check_is_at_least_1_32_0("alias")

        def resp(res: Response) -> bool:
            return res.status_code == 200

        return executor.execute(
            response_callback=resp,
            method=self._connection.get,
            path=f"/aliases/{alias_name}",
            error_msg="Alias may not exist.",
            status_codes=_ExpectedStatusCodes(ok_in=[200, 404], error="alias exists"),
        )



from typing import Any, Dict, List, Literal, Optional, Union

from deprecation import deprecated as docstring_deprecated
from pydantic import AnyHttpUrl, Field
from typing_extensions import deprecated as typing_deprecated

from weaviate.collections.classes.config_base import (
    _ConfigCreateModel,
    _ConfigUpdateModel,
    _EnumLikeStr,
)
from weaviate.collections.classes.config_vector_index import (
    VectorIndexType,
    _VectorIndexConfigCreate,
    _VectorIndexConfigDynamicUpdate,
    _VectorIndexConfigFlatUpdate,
    _VectorIndexConfigHNSWUpdate,
    _VectorIndexConfigUpdate,
)
from weaviate.collections.classes.config_vectorizers import (
    AWSModel,
    AWSService,
    CohereModel,
    CohereMultimodalModel,
    CohereTruncation,
    JinaModel,
    JinaMultimodalModel,
    Multi2VecField,
    OpenAIModel,
    OpenAIType,
    Vectorizers,
    VoyageModel,
    VoyageMultimodalModel,
    WeaviateModel,
    _Img2VecNeuralConfig,
    _map_multi2vec_fields,
    _Multi2VecBindConfig,
    _Multi2VecClipConfig,
    _Multi2VecCohereConfig,
    _Multi2VecGoogleConfig,
    _Multi2VecJinaConfig,
    _Multi2VecNvidiaConfig,
    _Multi2VecVoyageaiConfig,
    _Ref2VecCentroidConfig,
    _Text2ColbertJinaAIConfig,
    _Text2VecAWSConfig,
    _Text2VecAzureOpenAIConfig,
    _Text2VecCohereConfig,
    _Text2VecContextionaryConfig,
    _Text2VecDatabricksConfig,
    _Text2VecGoogleConfig,
    _Text2VecGPT4AllConfig,
    _Text2VecHuggingFaceConfig,
    _Text2VecJinaConfig,
    _Text2VecMistralConfig,
    _Text2VecModel2VecConfig,
    _Text2VecNvidiaConfig,
    _Text2VecOllamaConfig,
    _Text2VecOpenAIConfig,
    _Text2VecTransformersConfig,
    _Text2VecVoyageConfig,
    _Text2VecWeaviateConfig,
    _VectorizerConfigCreate,
    _VectorizerCustomConfig,
)

from ...warnings import _Warnings


class _NamedVectorConfigCreate(_ConfigCreateModel):
    name: str
    properties: Optional[List[str]] = Field(default=None, min_length=1, alias="source_properties")
    vectorizer: _VectorizerConfigCreate
    vectorIndexType: VectorIndexType = Field(default=VectorIndexType.HNSW, exclude=True)
    vectorIndexConfig: Optional[_VectorIndexConfigCreate] = Field(
        default=None, alias="vector_index_config"
    )

    def _to_dict(self) -> Dict[str, Any]:
        ret_dict: Dict[str, Any] = self.__parse_vectorizer()
        if self.vectorIndexConfig is not None:
            ret_dict["vectorIndexType"] = self.vectorIndexConfig.vector_index_type().value
            ret_dict["vectorIndexConfig"] = self.vectorIndexConfig._to_dict()
        else:
            ret_dict["vectorIndexType"] = self.vectorIndexType.value
        return ret_dict

    def __parse_vectorizer(self) -> Dict[str, Any]:
        vectorizer_options = self.vectorizer._to_dict()
        if self.properties is not None:
            vectorizer_options["properties"] = self.properties
        return {"vectorizer": {self.vectorizer.vectorizer.value: vectorizer_options}}


class _NamedVectorConfigUpdate(_ConfigUpdateModel):
    name: str
    vectorIndexConfig: _VectorIndexConfigUpdate = Field(..., alias="vector_index_config")


class _NamedVectors:
    @staticmethod
    def none(
        name: str, *, vector_index_config: Optional[_VectorIndexConfigCreate] = None
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using no vectorizer. You will need to provide the vectors yourself.

        Args:
            name: The name of the named vector.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
        """
        return _NamedVectorConfigCreate(
            name=name,
            vectorizer=_VectorizerConfigCreate(vectorizer=Vectorizers.NONE),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def custom(
        name: str,
        *,
        module_name: str,
        module_config: Optional[Dict[str, Any]] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using no vectorizer. You will need to provide the vectors yourself.

        Args:
            name: The name of the named vector.
            module_name: The name of the custom module to use.
            module_config: The configuration of the custom module to use.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_VectorizerCustomConfig(
                vectorizer=_EnumLikeStr(module_name), module_config=module_config
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2colbert_jinaai(
        name: str,
        *,
        dimensions: Optional[int] = None,
        model: Optional[str] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2colbert_jinaai` module.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/jinaai/colbert)
        for detailed usage.

        Args:
            name: The name of the named vector.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            dimensions: Number of dimensions. Applicable to v3 OpenAI models only. Defaults to `None`, which uses the server-defined default.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vector_index_config=vector_index_config,
            vectorizer=_Text2ColbertJinaAIConfig(
                model=model,
                dimensions=dimensions,
                vectorizeClassName=vectorize_collection_name,
            ),
        )

    @staticmethod
    def text2vec_cohere(
        name: str,
        *,
        base_url: Optional[AnyHttpUrl] = None,
        model: Optional[Union[CohereModel, str]] = None,
        truncate: Optional[CohereTruncation] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec_cohere` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/cohere/embeddings)
        for detailed usage.

        Args:
            name: The name of the named vector.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            truncate: The truncation strategy to use. Defaults to `None`, which uses the server-defined default.
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.

        Raises:
            pydantic.ValidationError: If `model` is not a valid value from the `CohereModel` type or if `truncate` is not a valid value from the `CohereTruncation` type.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecCohereConfig(
                baseURL=base_url,
                model=model,
                truncate=truncate,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def multi2vec_cohere(
        name: str,
        *,
        base_url: Optional[AnyHttpUrl] = None,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        model: Optional[Union[CohereMultimodalModel, str]] = None,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        truncate: Optional[CohereTruncation] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `multi2vec_cohere` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/cohere/embeddings-multimodal)
        for detailed usage.

        Args:
            name: The name of the named vector.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            truncate: The truncation strategy to use. Defaults to `None`, which uses the server-defined default.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            image_fields: The image fields to use in vectorization.
            text_fields: The text fields to use in vectorization.

        Raises:
            pydantic.ValidationError: If `model` is not a valid value from the `CohereMultimodalModel` type or if `truncate` is not a valid value from the `CohereTruncation` type.
        """
        return _NamedVectorConfigCreate(
            name=name,
            vectorizer=_Multi2VecCohereConfig(
                baseURL=base_url,
                model=model,
                truncate=truncate,
                imageFields=_map_multi2vec_fields(image_fields),
                textFields=_map_multi2vec_fields(text_fields),
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_contextionary(
        name: str,
        *,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec_contextionary` model.

        See the [documentation](https://weaviate.io/developers/weaviate/modules/retriever-vectorizer-modules/text2vec-contextionary)
        for detailed usage.

        Args:
            name: The name of the named vector.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecContextionaryConfig(
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_databricks(
        name: str,
        *,
        endpoint: str,
        instruction: Optional[str] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec-databricks` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/databricks/embeddings)
        for detailed usage.

        Args:
            name: The name of the named vector.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            endpoint: The endpoint to use.
            instruction: The instruction strategy to use. Defaults to `None`, which uses the server-defined default.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecDatabricksConfig(
                endpoint=endpoint,
                instruction=instruction,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_mistral(
        name: str,
        *,
        base_url: Optional[AnyHttpUrl] = None,
        model: Optional[str] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec-mistral` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/mistral/embeddings)
        for detailed usage.

        Args:
            name: The name of the named vector.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecMistralConfig(
                baseURL=base_url,
                model=model,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_ollama(
        name: str,
        *,
        api_endpoint: Optional[str] = None,
        model: Optional[str] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec-ollama` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/ollama/embeddings)
        for detailed usage.

        Args:
            name: The name of the named vector.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            api_endpoint: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
                Docker users may need to specify an alias, such as `http://host.docker.internal:11434` so that the container can access the host machine.

        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecOllamaConfig(
                apiEndpoint=api_endpoint,
                model=model,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_openai(
        name: str,
        *,
        base_url: Optional[AnyHttpUrl] = None,
        dimensions: Optional[int] = None,
        model: Optional[Union[OpenAIModel, str]] = None,
        model_version: Optional[str] = None,
        type_: Optional[OpenAIType] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec_openai` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/openai/embeddings)
        for detailed usage.

        Args:
            name: The name of the named vector.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            model_version: The model version to use. Defaults to `None`, which uses the server-defined default.
            type_: The type of model to use. Defaults to `None`, which uses the server-defined default.
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            dimensions: Number of dimensions. Applicable to v3 OpenAI models only. Defaults to `None`, which uses the server-defined default.

        Raises:
            pydantic.ValidationError: If `type_` is not a valid value from the `OpenAIType` type.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecOpenAIConfig(
                baseURL=base_url,
                model=model,
                modelVersion=model_version,
                type_=type_,
                vectorizeClassName=vectorize_collection_name,
                dimensions=dimensions,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_aws(
        name: str,
        region: str,
        *,
        endpoint: Optional[str] = None,
        model: Optional[Union[AWSModel, str]] = None,
        service: Union[AWSService, str] = "bedrock",
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec_aws` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/aws/embeddings)
        for detailed usage.

        Args:
            name: The name of the named vector.
            region: The AWS region to run the model from, REQUIRED.
            endpoint: The endpoint to use. Defaults to `None`, which uses the server-defined default.
            model: The model to use.
            service: The AWS service to use. Defaults to `bedrock`.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecAWSConfig(
                model=model,
                endpoint=endpoint,
                region=region,
                service=service,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def img2vec_neural(
        name: str,
        image_fields: List[str],
        *,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
    ) -> _NamedVectorConfigCreate:
        """Create a `Img2VecNeuralConfig` object for use when vectorizing using the `img2vec-neural` model.

        See the [documentation](https://weaviate.io/developers/weaviate/modules/retriever-vectorizer-modules/img2vec-neural)
        for detailed usage.

        Args:
            name: The name of the named vector.
            image_fields: The image fields to use. This is a required field and must match the property fields of the collection that are defined as `DataType.BLOB`.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default

        Raises:
            pydantic.ValidationError: If `image_fields` is not a `list`.
        """
        return _NamedVectorConfigCreate(
            name=name,
            vectorizer=_Img2VecNeuralConfig(imageFields=image_fields),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def multi2vec_clip(
        name: str,
        *,
        inference_url: Optional[str] = None,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `multi2vec_clip` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/transformers/embeddings-multimodal)
        for detailed usage.

        Args:
            name: The name of the named vector.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            image_fields: The image fields to use in vectorization.
            text_fields: The text fields to use in vectorization.
            inference_url: The inference url to use where API requests should go. Defaults to `None`, which uses the server-defined default.
        """
        return _NamedVectorConfigCreate(
            name=name,
            vectorizer=_Multi2VecClipConfig(
                imageFields=_map_multi2vec_fields(image_fields),
                textFields=_map_multi2vec_fields(text_fields),
                inferenceUrl=inference_url,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    @docstring_deprecated(
        deprecated_in="4.9.0",
        details="""
This method is deprecated and will be removed in Q2 '25. Please use :meth:`~weaviate.collections.classes.config._NamedVectors.multi2vec_google` instead.
""",
    )
    @typing_deprecated(
        "This method is deprecated and will be removed in Q2 '25. Please use `multi2vec_google` instead."
    )
    def multi2vec_palm(
        name: str,
        *,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
        location: str,
        project_id: str,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        video_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        dimensions: Optional[int] = None,
        video_interval_seconds: Optional[int] = None,
        model_id: Optional[str] = None,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `multi2vec_palm` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/google/embeddings-multimodal)
        for detailed usage.

        Args:
            name: The name of the named vector.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            location: Where the model runs. REQUIRED.
            project_id: The project ID to use, REQUIRED.
            image_fields: The image fields to use in vectorization.
            text_fields: The text fields to use in vectorization.
            video_fields: The video fields to use in vectorization.
            dimensions: The number of dimensions to use. Defaults to `None`, which uses the server-defined default.
            video_interval_seconds: Length of a video interval. Defaults to `None`, which uses the server-defined default.
            model_id: The model ID to use. Defaults to `None`, which uses the server-defined default.
        """
        _Warnings.palm_to_google_m2v()
        return _NamedVectorConfigCreate(
            name=name,
            vectorizer=_Multi2VecGoogleConfig(
                projectId=project_id,
                location=location,
                imageFields=_map_multi2vec_fields(image_fields),
                textFields=_map_multi2vec_fields(text_fields),
                videoFields=_map_multi2vec_fields(video_fields),
                dimensions=dimensions,
                modelId=model_id,
                videoIntervalSeconds=video_interval_seconds,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def multi2vec_google(
        name: str,
        *,
        location: str,
        project_id: str,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        video_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        dimensions: Optional[int] = None,
        video_interval_seconds: Optional[int] = None,
        model_id: Optional[str] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `multi2vec_google` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/google/embeddings-multimodal)
        for detailed usage.

        Args:
            name: The name of the named vector.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            location: Where the model runs. REQUIRED.
            project_id: The project ID to use, REQUIRED.
            image_fields: The image fields to use in vectorization.
            text_fields: The text fields to use in vectorization.
            video_fields: The video fields to use in vectorization.
            dimensions: The number of dimensions to use. Defaults to `None`, which uses the server-defined default.
            video_interval_seconds: Length of a video interval. Defaults to `None`, which uses the server-defined default.
            model_id: The model ID to use. Defaults to `None`, which uses the server-defined default.
        """
        return _NamedVectorConfigCreate(
            name=name,
            vectorizer=_Multi2VecGoogleConfig(
                projectId=project_id,
                location=location,
                imageFields=_map_multi2vec_fields(image_fields),
                textFields=_map_multi2vec_fields(text_fields),
                videoFields=_map_multi2vec_fields(video_fields),
                dimensions=dimensions,
                modelId=model_id,
                videoIntervalSeconds=video_interval_seconds,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def multi2vec_bind(
        name: str,
        *,
        audio_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        depth_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        imu_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        thermal_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        video_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `multi2vec_bind` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/imagebind/embeddings-multimodal)
        for detailed usage.

        Args:
            name: The name of the named vector.
            audio_fields: The audio fields to use in vectorization.
            depth_fields: The depth fields to use in vectorization.
            image_fields: The image fields to use in vectorization.
            imu_fields: The IMU fields to use in vectorization.
            text_fields: The text fields to use in vectorization.
            thermal_fields: The thermal fields to use in vectorization.
            video_fields: The video fields to use in vectorization.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _NamedVectorConfigCreate(
            name=name,
            vectorizer=_Multi2VecBindConfig(
                audioFields=_map_multi2vec_fields(audio_fields),
                depthFields=_map_multi2vec_fields(depth_fields),
                imageFields=_map_multi2vec_fields(image_fields),
                IMUFields=_map_multi2vec_fields(imu_fields),
                textFields=_map_multi2vec_fields(text_fields),
                thermalFields=_map_multi2vec_fields(thermal_fields),
                videoFields=_map_multi2vec_fields(video_fields),
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def multi2vec_voyageai(
        name: str,
        *,
        base_url: Optional[AnyHttpUrl] = None,
        model: Optional[Union[VoyageMultimodalModel, str]] = None,
        truncation: Optional[bool] = None,
        output_encoding: Optional[str] = None,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `multi2vec_voyageai` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/voyageai/embeddings-multimodal)
        for detailed usage.

        Args:
            name: The name of the named vector.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            truncation: The truncation strategy to use. Defaults to `None`, which uses the server-defined default.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            image_fields: The image fields to use in vectorization.
            text_fields: The text fields to use in vectorization.

        Raises:
            pydantic.ValidationError: If `model` is not a valid value from the `VoyageaiMultimodalModel` type.
        """
        return _NamedVectorConfigCreate(
            name=name,
            vectorizer=_Multi2VecVoyageaiConfig(
                baseURL=base_url,
                model=model,
                truncation=truncation,
                imageFields=_map_multi2vec_fields(image_fields),
                textFields=_map_multi2vec_fields(text_fields),
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def multi2vec_nvidia(
        name: str,
        *,
        base_url: Optional[AnyHttpUrl] = None,
        model: Optional[str] = None,
        truncation: Optional[bool] = None,
        output_encoding: Optional[str] = None,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `multi2vec_nvidia` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/nvidia/embeddings-multimodal)
        for detailed usage.

        Args:
            name: The name of the named vector.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            truncation: The truncation strategy to use. Defaults to `None`, which uses the server-defined default.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            image_fields: The image fields to use in vectorization.
            text_fields: The text fields to use in vectorization.

        Raises:
            pydantic.ValidationError: If `model` is not a valid value from the `NvidiaMultimodalModel` type.
        """
        return _NamedVectorConfigCreate(
            name=name,
            vectorizer=_Multi2VecNvidiaConfig(
                baseURL=base_url,
                model=model,
                truncation=truncation,
                imageFields=_map_multi2vec_fields(image_fields),
                textFields=_map_multi2vec_fields(text_fields),
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def ref2vec_centroid(
        name: str,
        reference_properties: List[str],
        *,
        method: Literal["mean"] = "mean",
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `ref2vec_centroid` model.

        See the [documentation](https://weaviate.io/developers/weaviate/modules/retriever-vectorizer-modules/text2vec-gpt4all)
        for detailed usage.

        Args:
            name: The name of the named vector.
            reference_properties: The reference properties to use in vectorization, REQUIRED.
            method: The method to use. Defaults to `mean`.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
        """
        return _NamedVectorConfigCreate(
            name=name,
            vectorizer=_Ref2VecCentroidConfig(
                referenceProperties=reference_properties,
                method=method,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_azure_openai(
        name: str,
        resource_name: str,
        deployment_id: str,
        *,
        base_url: Optional[AnyHttpUrl] = None,
        dimensions: Optional[int] = None,
        model: Optional[str] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec_azure_openai` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/openai-azure/embeddings)
        for detailed usage.

        Args:
            name: The name of the named vector.
            resource_name: The resource name to use, REQUIRED.
            deployment_id: The deployment ID to use, REQUIRED.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            dimensions: The dimensionality of the vectors. Defaults to `None`, which uses the server-defined default.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecAzureOpenAIConfig(
                baseURL=base_url,
                dimensions=dimensions,
                model=model,
                resourceName=resource_name,
                deploymentId=deployment_id,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_gpt4all(
        name: str,
        *,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec_gpt4all` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/gpt4all/embeddings)
        for detailed usage.

        Args:
            name: The name of the named vector.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecGPT4AllConfig(
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_huggingface(
        name: str,
        *,
        model: Optional[str] = None,
        passage_model: Optional[str] = None,
        query_model: Optional[str] = None,
        endpoint_url: Optional[AnyHttpUrl] = None,
        wait_for_model: Optional[bool] = None,
        use_gpu: Optional[bool] = None,
        use_cache: Optional[bool] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec_huggingface` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/huggingface/embeddings)
        for detailed usage.

        Args:
            name: The name of the named vector.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            passage_model: The passage model to use. Defaults to `None`, which uses the server-defined default.
            query_model: The query model to use. Defaults to `None`, which uses the server-defined default.
            endpoint_url: The endpoint URL to use. Defaults to `None`, which uses the server-defined default.
            wait_for_model: Whether to wait for the model to be loaded. Defaults to `None`, which uses the server-defined default.
            use_gpu: Whether to use the GPU. Defaults to `None`, which uses the server-defined default.
            use_cache: Whether to use the cache. Defaults to `None`, which uses the server-defined default.

        Raises:
            pydantic.ValidationError: If the arguments passed to the function are invalid.
                It is important to note that some of these variables are mutually exclusive.
                See the [documentation](https://weaviate.io/developers/weaviate/model-providers/huggingface/embeddings#vectorizer-parameters) for more details.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecHuggingFaceConfig(
                model=model,
                passageModel=passage_model,
                queryModel=query_model,
                endpointURL=endpoint_url,
                waitForModel=wait_for_model,
                useGPU=use_gpu,
                useCache=use_cache,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    @docstring_deprecated(
        deprecated_in="4.9.0",
        details="""
This method is deprecated and will be removed in Q2 '25. Please use :meth:`~weaviate.collections.classes.config._NamedVectors.text2vec_google` instead.
""",
    )
    @typing_deprecated(
        "This method is deprecated and will be removed in Q2 '25. Please use `text2vec_google` instead."
    )
    def text2vec_palm(
        name: str,
        project_id: str,
        *,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
        api_endpoint: Optional[str] = None,
        model_id: Optional[str] = None,
        title_property: Optional[str] = None,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec_palm` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/google/embeddings)
        for detailed usage.

        Args:
            name: The name of the named vector.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            project_id: The project ID to use, REQUIRED.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            api_endpoint: The API endpoint to use without a leading scheme such as `http://`. Defaults to `None`, which uses the server-defined default
            model_id: The model ID to use. Defaults to `None`, which uses the server-defined default.
            title_property: The Weaviate property name for the `gecko-002` or `gecko-003` model to use as the title.

        Raises:
            pydantic.ValidationError: If `api_endpoint` is not a valid URL.
        """
        _Warnings.palm_to_google_t2v()
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecGoogleConfig(
                projectId=project_id,
                apiEndpoint=api_endpoint,
                dimensions=None,
                modelId=model_id,
                vectorizeClassName=vectorize_collection_name,
                titleProperty=title_property,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_google(
        name: str,
        project_id: str,
        *,
        api_endpoint: Optional[str] = None,
        model_id: Optional[str] = None,
        title_property: Optional[str] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec_palm` model.

        See the [documentation]https://weaviate.io/developers/weaviate/model-providers/google/embeddings)
        for detailed usage.

        Args:
            name: The name of the named vector.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            project_id: The project ID to use, REQUIRED.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            api_endpoint: The API endpoint to use without a leading scheme such as `http://`. Defaults to `None`, which uses the server-defined default
            model_id: The model ID to use. Defaults to `None`, which uses the server-defined default.
            title_property: The Weaviate property name for the `gecko-002` or `gecko-003` model to use as the title.

        Raises:
            pydantic.ValidationError: If `api_endpoint` is not a valid URL.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecGoogleConfig(
                projectId=project_id,
                apiEndpoint=api_endpoint,
                dimensions=None,
                modelId=model_id,
                vectorizeClassName=vectorize_collection_name,
                titleProperty=title_property,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_google_aistudio(
        name: str,
        *,
        model_id: Optional[str] = None,
        title_property: Optional[str] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec_palm` model.

        See the [documentation]https://weaviate.io/developers/weaviate/model-providers/google/embeddings)
        for detailed usage.

        Args:
            name: The name of the named vector.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            model_id: The model ID to use. Defaults to `None`, which uses the server-defined default.
            title_property: The Weaviate property name for the `gecko-002` or `gecko-003` model to use as the title.

        Raises:
            pydantic.ValidationError: If `api_endpoint` is not a valid URL.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecGoogleConfig(
                projectId=None,
                apiEndpoint="generativelanguage.googleapis.com",
                dimensions=None,
                modelId=model_id,
                vectorizeClassName=vectorize_collection_name,
                titleProperty=title_property,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_transformers(
        name: str,
        *,
        dimensions: Optional[int] = None,
        pooling_strategy: Literal["masked_mean", "cls"] = "masked_mean",
        inference_url: Optional[str] = None,
        passage_inference_url: Optional[str] = None,
        query_inference_url: Optional[str] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec_transformers` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/transformers/embeddings)
        for detailed usage.

        Args:
            name: The name of the named vector.
            dimensions: The number of dimensions for the generated embeddings. Defaults to `None`, which uses the server-defined default.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            pooling_strategy: The pooling strategy to use. Defaults to `masked_mean`.
            inference_url: The inferenceUrl to use where API requests should go. You can use either this OR passage/query_inference_url. Defaults to `None`, which uses the server-defined default.
            passage_inference_url: The inferenceUrl to use where passage API requests should go. You can use either this and query_inference_url OR inference_url. Defaults to `None`, which uses the server-defined default.
            query_inference_url: The inferenceUrl to use where query API requests should go. You can use either this and passage_inference_url OR inference_url. Defaults to `None`, which uses the server-defined default.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecTransformersConfig(
                dimensions=dimensions,
                poolingStrategy=pooling_strategy,
                vectorizeClassName=vectorize_collection_name,
                inferenceUrl=inference_url,
                passageInferenceUrl=passage_inference_url,
                queryInferenceUrl=query_inference_url,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_jinaai(
        name: str,
        *,
        base_url: Optional[str] = None,
        dimensions: Optional[int] = None,
        model: Optional[Union[JinaModel, str]] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec-jinaai` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/jinaai/embeddings) for detailed usage.

        Args:
            name: The name of the named vector.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            base_url: The base URL to send the vectorization requests to. Defaults to `None`, which uses the server-defined default.
            dimensions: The number of dimensions for the generated embeddings. Defaults to `None`, which uses the server-defined default.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecJinaConfig(
                baseURL=base_url,
                dimensions=dimensions,
                model=model,
                vectorizeClassName=vectorize_collection_name,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def multi2vec_jinaai(
        name: str,
        *,
        base_url: Optional[AnyHttpUrl] = None,
        model: Optional[Union[JinaMultimodalModel, str]] = None,
        dimensions: Optional[int] = None,
        image_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        text_fields: Optional[Union[List[str], List[Multi2VecField]]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `multi2vec_jinaai` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/jinaai/embeddings-multimodal)
        for detailed usage.

        Args:
            name: The name of the named vector.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            dimensions: The number of dimensions for the generated embeddings (only available for some models). Defaults to `None`, which uses the server-defined default.
            image_fields: The image fields to use in vectorization.
            text_fields: The text fields to use in vectorization.

        Raises:
            pydantic.ValidationError: If `model` is not a valid value from the `JinaMultimodalModel` type.
        """
        return _NamedVectorConfigCreate(
            name=name,
            vectorizer=_Multi2VecJinaConfig(
                baseURL=base_url,
                model=model,
                dimensions=dimensions,
                imageFields=_map_multi2vec_fields(image_fields),
                textFields=_map_multi2vec_fields(text_fields),
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_voyageai(
        name: str,
        *,
        model: Optional[Union[VoyageModel, str]] = None,
        base_url: Optional[str] = None,
        truncate: Optional[bool] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec-jinaai` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/voyageai/embeddings)
        for detailed usage.

        Args:
            name: The name of the named vector.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
                See the
                [documentation](https://weaviate.io/developers/weaviate/model-providers/voyageai/embeddings#available-models) for more details.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            truncate: Whether to truncate the input texts to fit within the context length. Defaults to `None`, which uses the server-defined default.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecVoyageConfig(
                model=model,
                vectorizeClassName=vectorize_collection_name,
                baseURL=base_url,
                truncate=truncate,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_weaviate(
        name: str,
        *,
        model: Optional[Union[WeaviateModel, str]] = None,
        base_url: Optional[str] = None,
        dimensions: Optional[int] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecWeaviateConfig(
                model=model,
                vectorizeClassName=vectorize_collection_name,
                baseURL=base_url,
                dimensions=dimensions,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_nvidia(
        name: str,
        *,
        model: Optional[str] = None,
        base_url: Optional[str] = None,
        truncate: Optional[bool] = None,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec-nvidia` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/nvidia/embeddings)
        for detailed usage.

        Args:
            name: The name of the named vector.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            model: The model to use. Defaults to `None`, which uses the server-defined default.
                See the
                [documentation](https://weaviate.io/developers/weaviate/model-providers/nvidia/embeddings#available-models) for more details.
            base_url: The base URL to use where API requests should go. Defaults to `None`, which uses the server-defined default.
            truncate: Whether to truncate the input texts to fit within the context length. Defaults to `None`, which uses the server-defined default.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecNvidiaConfig(
                model=model,
                vectorizeClassName=vectorize_collection_name,
                baseURL=base_url,
                truncate=truncate,
            ),
            vector_index_config=vector_index_config,
        )

    @staticmethod
    def text2vec_model2vec(
        name: str,
        *,
        source_properties: Optional[List[str]] = None,
        vector_index_config: Optional[_VectorIndexConfigCreate] = None,
        vectorize_collection_name: bool = True,
        inference_url: Optional[str] = None,
    ) -> _NamedVectorConfigCreate:
        """Create a named vector using the `text2vec-model2vec` model.

        See the [documentation](https://weaviate.io/developers/weaviate/model-providers/model2vec/embeddings)
        for detailed usage.

        Args:
            name: The name of the named vector.
            source_properties: Which properties should be included when vectorizing. By default all text properties are included.
            vector_index_config: The configuration for Weaviate's vector index. Use wvc.config.Configure.VectorIndex to create a vector index configuration. None by default
            vectorize_collection_name: Whether to vectorize the collection name. Defaults to `True`.
            inference_url: The inference url to use where API requests should go. Defaults to `None`, which uses the server-defined default.
        """
        return _NamedVectorConfigCreate(
            name=name,
            source_properties=source_properties,
            vectorizer=_Text2VecModel2VecConfig(
                vectorizeClassName=vectorize_collection_name,
                inferenceUrl=inference_url,
            ),
            vector_index_config=vector_index_config,
        )


class _NamedVectorsUpdate:
    @staticmethod
    def update(
        name: str,
        *,
        vector_index_config: Union[
            _VectorIndexConfigHNSWUpdate,
            _VectorIndexConfigFlatUpdate,
            _VectorIndexConfigDynamicUpdate,
        ],
    ) -> _NamedVectorConfigUpdate:
        """Update the vector index configuration of a named vector.

        This is the only update operation allowed currently. If you wish to change the vectorization configuration itself, you will have to
        recreate the collection with the new configuration.

        Args:
            name: The name of the named vector.
            vector_index_config: The configuration for Weaviate's vector index. Use `wvc.config.Reconfigure.VectorIndex` to create a vector index configuration. `None` by default
        """
        return _NamedVectorConfigUpdate(
            name=name,
            vector_index_config=vector_index_config,
        )



from typing import Dict, List, Optional

from httpx import Response

from weaviate.collections.classes.batch import (
    BatchReference,
    BatchReferenceReturn,
    ErrorReference,
    _BatchReference,
)
from weaviate.collections.classes.config import ConsistencyLevel
from weaviate.connect import executor
from weaviate.connect.v4 import Connection, _ExpectedStatusCodes
from weaviate.util import _decode_json_response_list


class _BatchREST:
    def __init__(self, consistency_level: Optional[ConsistencyLevel]) -> None:
        self.__consistency_level = consistency_level

    def references(
        self, connection: Connection, *, references: List[_BatchReference]
    ) -> executor.Result[BatchReferenceReturn]:
        params: Dict[str, str] = {}
        if self.__consistency_level is not None:
            params["consistency_level"] = self.__consistency_level.value

        refs = [
            (
                {"from": ref.from_, "to": ref.to}
                if ref.tenant is None
                else {"from": ref.from_, "to": ref.to, "tenant": ref.tenant}
            )
            for ref in references
        ]

        def resp(res: Response) -> BatchReferenceReturn:
            payload = _decode_json_response_list(res, "batch ref")
            assert payload is not None
            errors = {
                idx: ErrorReference(
                    message=entry["result"]["errors"]["error"][0]["message"],
                    reference=BatchReference._from_internal(references[idx]),
                )
                for idx, entry in enumerate(payload)
                if entry["result"]["status"] == "FAILED"
            }
            return BatchReferenceReturn(
                elapsed_seconds=res.elapsed.total_seconds(),
                errors=errors,
                has_errors=len(errors) > 0,
            )

        return executor.execute(
            response_callback=resp,
            method=connection.post,
            path="/batch/references",
            weaviate_object=refs,
            params=params,
            status_codes=_ExpectedStatusCodes(ok_in=200, error="Send ref batch"),
        )



from weaviate.collections.aggregations.near_image.executor import _NearImageExecutor
from weaviate.connect import executor
from weaviate.connect.v4 import ConnectionSync


@executor.wrap("sync")
class _NearImage(_NearImageExecutor[ConnectionSync]):
    pass



import uuid
from typing import Generic, List, Optional, Union

from httpx import Response

from weaviate.cluster.models import (
    ReplicationType,
    ShardingState,
)
from weaviate.cluster.types import Verbosity
from weaviate.collections.classes.cluster import NodeMinimal, NodeVerbose, _ConvertFromREST
from weaviate.connect import executor
from weaviate.connect.v4 import ConnectionType, _ExpectedStatusCodes
from weaviate.exceptions import EmptyResponseError
from weaviate.util import _capitalize_first_letter, _decode_json_response_dict


class _ClusterExecutor(Generic[ConnectionType]):
    def __init__(self, connection: ConnectionType):
        self._connection = connection

    def replicate(
        self,
        *,
        collection: str,
        shard: str,
        source_node: str,
        target_node: str,
        replication_type: ReplicationType = ReplicationType.COPY,
    ) -> executor.Result[uuid.UUID]:
        """Replicate a shard from one node to another.

        Args:
            collection: The name of the collection.
            shard: The name of the shard.
            source_node: The source node.
            target_node: The target node.
            replication_type: The type of replication (COPY or MOVE).

        Returns:
            A UUID representing the replicate task.
        """

        def resp(response: Response):
            return uuid.UUID(response.json()["id"])

        body = {
            "collection": collection,
            "shard": shard,
            "sourceNode": source_node,
            "targetNode": target_node,
            "type": replication_type.value,
        }
        return executor.execute(
            response_callback=resp,
            method=self._connection.post,
            path="/replication/replicate",
            weaviate_object=body,
            status_codes=_ExpectedStatusCodes(200, "replicate replicate"),
            error_msg="Failed to replicate shard",
        )

    def query_sharding_state(
        self,
        *,
        collection: str,
        shard: Optional[str] = None,
    ) -> executor.Result[Optional[ShardingState]]:
        """Query the sharding state of a collection or shard.

        If shard is None, the state of all shards in the collection will be returned.

        Args:
            collection: The name of the collection.
            shard: The name of the shard.

        Returns:
            The sharding state or None if the collection or shard does not exist.
        """

        def resp(response: Response):
            if response.status_code == 404:
                return None
            return ShardingState._from_weaviate(response.json())

        params = {"collection": collection}
        if shard is not None:
            params["shard"] = shard

        return executor.execute(
            response_callback=resp,
            method=self._connection.get,
            path="/replication/sharding-state",
            params=params,
            status_codes=_ExpectedStatusCodes([200, 404], "replicate sharding state"),
            error_msg="Failed to get sharding state",
        )

    def nodes(
        self,
        collection: Optional[str] = None,
        shard: Optional[str] = None,
        *,
        output: Optional[Verbosity] = None,
    ) -> executor.Result[Union[List[NodeMinimal], List[NodeVerbose]]]:
        """Get the status of all nodes in the cluster.

        Args:
            collection: Get the status for the given collection. If not given all collections will be included.
            shard: Get the status for the given shard. If not given all shards will be included.
            output: Set the desired output verbosity level. Can be [`minimal` | `verbose`], defaults to `None`, which is server-side default of `minimal`.

        Returns:
            List of nodes and their respective status.

        Raises:
            weaviate.exceptions.WeaviateConnectionError: If the network connection to weaviate fails.
            weaviate.exceptions.UnexpectedStatusCodeError: If weaviate reports a none OK status.
            weaviate.EmptyResponseError: If the response is empty.
        """
        path = "/nodes"
        params = {}
        if collection is not None:
            path += "/" + _capitalize_first_letter(collection)
        if shard is not None:
            params["shardName"] = shard
        if output is not None:
            params["output"] = output

        def resp(
            res: Response,
        ) -> Union[List[NodeMinimal], List[NodeVerbose]]:
            response_typed = _decode_json_response_dict(res, "Nodes status")
            assert response_typed is not None

            nodes = response_typed.get("nodes")
            if nodes is None or nodes == []:
                raise EmptyResponseError("Nodes status response returned empty")

            if output == "verbose":
                return _ConvertFromREST.nodes_verbose(nodes)
            else:
                return _ConvertFromREST.nodes_minimal(nodes)

        return executor.execute(
            response_callback=resp,
            method=self._connection.get,
            path=path,
            params=params,
            error_msg="Get nodes status failed",
        )



from weaviate.aliases.executor import _AliasExecutor
from weaviate.connect import executor
from weaviate.connect.v4 import ConnectionSync


@executor.wrap("sync")
class _Alias(_AliasExecutor[ConnectionSync]):
    pass



from typing import Generic

from weaviate.collections.classes.types import References, TProperties
from weaviate.collections.queries.bm25 import _BM25Query, _BM25QueryAsync
from weaviate.collections.queries.fetch_object_by_id import (
    _FetchObjectByIDQuery,
    _FetchObjectByIDQueryAsync,
)
from weaviate.collections.queries.fetch_objects import (
    _FetchObjectsQuery,
    _FetchObjectsQueryAsync,
)
from weaviate.collections.queries.fetch_objects_by_ids import (
    _FetchObjectsByIDsQuery,
    _FetchObjectsByIDsQueryAsync,
)
from weaviate.collections.queries.hybrid import _HybridQuery, _HybridQueryAsync
from weaviate.collections.queries.near_image import (
    _NearImageQuery,
    _NearImageQueryAsync,
)
from weaviate.collections.queries.near_media import (
    _NearMediaQuery,
    _NearMediaQueryAsync,
)
from weaviate.collections.queries.near_object import (
    _NearObjectQuery,
    _NearObjectQueryAsync,
)
from weaviate.collections.queries.near_text import _NearTextQuery, _NearTextQueryAsync
from weaviate.collections.queries.near_vector import (
    _NearVectorQuery,
    _NearVectorQueryAsync,
)


class _QueryCollectionAsync(
    Generic[TProperties, References],
    _BM25QueryAsync[TProperties, References],
    _FetchObjectByIDQueryAsync[TProperties, References],
    _FetchObjectsByIDsQueryAsync[TProperties, References],
    _FetchObjectsQueryAsync[TProperties, References],
    _HybridQueryAsync[TProperties, References],
    _NearImageQueryAsync[TProperties, References],
    _NearMediaQueryAsync[TProperties, References],
    _NearObjectQueryAsync[TProperties, References],
    _NearTextQueryAsync[TProperties, References],
    _NearVectorQueryAsync[TProperties, References],
):
    pass


class _QueryCollection(
    Generic[TProperties, References],
    _BM25Query[TProperties, References],
    _FetchObjectByIDQuery[TProperties, References],
    _FetchObjectsByIDsQuery[TProperties, References],
    _FetchObjectsQuery[TProperties, References],
    _HybridQuery[TProperties, References],
    _NearImageQuery[TProperties, References],
    _NearMediaQuery[TProperties, References],
    _NearObjectQuery[TProperties, References],
    _NearTextQuery[TProperties, References],
    _NearVectorQuery[TProperties, References],
):
    pass



from .generate import _NearImageGenerate, _NearImageGenerateAsync
from .query import _NearImageQuery, _NearImageQueryAsync

__all__ = [
    "_NearImageGenerate",
    "_NearImageQuery",
    "_NearImageGenerateAsync",
    "_NearImageQueryAsync",
]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/health_weaviate.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18v1/health_weaviate.proto\x12\x0bweaviate.v1\"-\n\x1aWeaviateHealthCheckRequest\x12\x0f\n\x07service\x18\x01 \x01(\t\"\xa1\x01\n\x1bWeaviateHealthCheckResponse\x12\x46\n\x06status\x18\x01 \x01(\x0e\x32\x36.weaviate.v1.WeaviateHealthCheckResponse.ServingStatus\":\n\rServingStatus\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0b\n\x07SERVING\x10\x01\x12\x0f\n\x0bNOT_SERVING\x10\x02\x42k\n#io.weaviate.client.grpc.protocol.v1B\x0eWeaviateHealthZ4github.com/weaviate/weaviate/grpc/generated;protocolb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'v1.health_weaviate_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n#io.weaviate.client.grpc.protocol.v1B\016WeaviateHealthZ4github.com/weaviate/weaviate/grpc/generated;protocol'
  _globals['_WEAVIATEHEALTHCHECKREQUEST']._serialized_start=41
  _globals['_WEAVIATEHEALTHCHECKREQUEST']._serialized_end=86
  _globals['_WEAVIATEHEALTHCHECKRESPONSE']._serialized_start=89
  _globals['_WEAVIATEHEALTHCHECKRESPONSE']._serialized_end=250
  _globals['_WEAVIATEHEALTHCHECKRESPONSE_SERVINGSTATUS']._serialized_start=192
  _globals['_WEAVIATEHEALTHCHECKRESPONSE_SERVINGSTATUS']._serialized_end=250
# @@protoc_insertion_point(module_scope)



from integration.conftest import CollectionFactory
from weaviate.collections.classes.config import Configure, DataType, Property
from weaviate.collections.classes.filters import Filter
from weaviate.collections.classes.types import GeoCoordinate


def test_creating_geo_props(collection_factory: CollectionFactory) -> None:
    collection = collection_factory(
        vectorizer_config=Configure.Vectorizer.none(),
        properties=[Property(name="geo", data_type=DataType.GEO_COORDINATES)],
    )

    obj_uuid = collection.data.insert({"geo": GeoCoordinate(latitude=1.0, longitude=1.0)})

    obj = collection.query.fetch_object_by_id(obj_uuid)
    assert obj is not None
    assert obj.properties["geo"] == GeoCoordinate(latitude=1.0, longitude=1.0)

    batch_ret = collection.data.insert_many([{"geo": GeoCoordinate(latitude=2.0, longitude=1.0)}])
    assert not batch_ret.has_errors

    obj_batch = collection.query.fetch_object_by_id(batch_ret.uuids[0])
    assert obj_batch is not None
    assert obj_batch.properties["geo"] == GeoCoordinate(latitude=2.0, longitude=1.0)

    # also accept dicts, but don't show that in the docs
    obj_uuid2 = collection.data.insert({"geo": {"latitude": 1.0, "longitude": 3.0}})
    obj2 = collection.query.fetch_object_by_id(obj_uuid2)
    assert obj2 is not None
    assert obj2.properties["geo"] == GeoCoordinate(latitude=1.0, longitude=3.0)

    batch_ret2 = collection.data.insert_many([{"geo": GeoCoordinate(latitude=3.0, longitude=1.0)}])
    assert not batch_ret2.has_errors

    obj_batch2 = collection.query.fetch_object_by_id(batch_ret2.uuids[0])
    assert obj_batch2 is not None
    assert obj_batch2.properties["geo"] == GeoCoordinate(latitude=3.0, longitude=1.0)


def test_geo_props_query(collection_factory: CollectionFactory) -> None:
    collection = collection_factory(
        vectorizer_config=Configure.Vectorizer.none(),
        properties=[Property(name="geo", data_type=DataType.GEO_COORDINATES)],
    )

    collection.data.insert({"geo": GeoCoordinate(latitude=1.0, longitude=2.0)})

    objs = collection.query.fetch_objects()
    assert objs.objects[0].properties["geo"] == GeoCoordinate(latitude=1.0, longitude=2.0)


def test_geo_props_filter(collection_factory: CollectionFactory) -> None:
    collection = collection_factory(
        vectorizer_config=Configure.Vectorizer.none(),
        properties=[Property(name="geo", data_type=DataType.GEO_COORDINATES)],
    )

    uuid1 = collection.data.insert({"geo": GeoCoordinate(latitude=1.0, longitude=2.0)})
    collection.data.insert({"geo": GeoCoordinate(latitude=89.0, longitude=179.0)})
    objs = collection.query.fetch_objects(
        filters=Filter.by_property("geo").within_geo_range(
            GeoCoordinate(latitude=1.0, longitude=2.01), distance=2000.0
        ),
        return_properties=["geo"],
    )
    assert len(objs.objects) == 1
    assert objs.objects[0].uuid == uuid1
    assert objs.objects[0].properties["geo"] == GeoCoordinate(latitude=1.0, longitude=2.0)



from typing import Generic, Literal, Optional, overload

from httpx import Response

from weaviate.cluster.models import (
    ReplicateOperation,
    ReplicateOperations,
    ReplicateOperationWithHistory,
    ReplicateOperationWithoutHistory,
    _ReplicateOperation,
)
from weaviate.connect import executor
from weaviate.connect.v4 import ConnectionType, _ExpectedStatusCodes
from weaviate.types import UUID


class _ReplicateExecutor(Generic[ConnectionType]):
    def __init__(self, connection: ConnectionType):
        self._connection = connection

    @overload
    def get(
        self, *, uuid: UUID, include_history: Literal[False] = False
    ) -> executor.Result[Optional[ReplicateOperationWithoutHistory]]: ...

    @overload
    def get(
        self, *, uuid: UUID, include_history: Literal[True]
    ) -> executor.Result[Optional[ReplicateOperationWithHistory]]: ...

    def get(
        self, *, uuid: UUID, include_history: bool = False
    ) -> executor.Result[Optional[ReplicateOperation]]:
        """Get the a replicate operation by its UUID.

        Args:
            uuid: The ID of the replicate operation.
            include_history: Whether to include the history of the operation.

        Returns:
            The replicate operation.
        """

        def resp(response: Response):
            if response.status_code == 404:
                return None
            return _ReplicateOperation._from_weaviate(response.json(), include_history)

        params = {}
        if include_history:
            params["includeHistory"] = include_history

        return executor.execute(
            response_callback=resp,
            method=self._connection.get,
            path=f"/replication/replicate/{uuid}",
            params=params,
            status_codes=_ExpectedStatusCodes([200, 404], "replicate get"),
            error_msg="Failed to get replicate operation",
        )

    def list_all(self) -> executor.Result[list[ReplicateOperationWithHistory]]:
        """List all replicate operations.

        Returns:
            A list of replicate operations.
        """

        def resp(response: Response) -> list[ReplicateOperationWithHistory]:
            return [_ReplicateOperation._from_weaviate(item, True) for item in response.json()]  # pyright: ignore[reportReturnType]

        return executor.execute(
            response_callback=resp,
            method=self._connection.get,
            path="/replication/replicate/list",
            params={"includeHistory": True},
            status_codes=_ExpectedStatusCodes(200, "replicate list"),
            error_msg="Failed to list replicate operations",
        )

    @overload
    def query(
        self,
        *,
        collection: Optional[str] = None,
        shard: Optional[str] = None,
        target_node: Optional[str] = None,
        include_history: Literal[True],
    ) -> executor.Result[list[ReplicateOperationWithHistory]]: ...

    @overload
    def query(
        self,
        *,
        collection: Optional[str] = None,
        shard: Optional[str] = None,
        target_node: Optional[str] = None,
        include_history: Literal[False] = False,
    ) -> executor.Result[list[ReplicateOperationWithoutHistory]]: ...

    def query(
        self,
        *,
        collection: Optional[str] = None,
        shard: Optional[str] = None,
        target_node: Optional[str] = None,
        include_history: bool = False,
    ) -> executor.Result[ReplicateOperations]:
        """Query replicate operations by collection, shard, node, or any combination of the three.

        Args:
            collection: The name of the collection of the operation.
            shard: The name of the shard of the operation.
            target_node: The name of the target node of the operation.
            include_history: Whether to include the history of the operation.

        Returns:
            A list of replicate operations specific to the provided parameters.
        """

        def resp(response: Response) -> ReplicateOperations:
            return [
                _ReplicateOperation._from_weaviate(item, include_history)
                for item in response.json()  # pyright: ignore[reportReturnType]
            ]

        params = {}
        if collection:
            params["collection"] = collection
        if shard:
            params["shard"] = shard
        if target_node:
            params["targetNode"] = target_node
        if include_history:
            params["includeHistory"] = include_history

        return executor.execute(
            response_callback=resp,
            method=self._connection.get,
            path="/replication/replicate/list",
            status_codes=_ExpectedStatusCodes(200, "replicate query"),
            error_msg="Failed to query replicate operations",
            params=params,
        )

    def cancel(
        self,
        *,
        uuid: UUID,
    ) -> executor.Result[None]:
        """Cancel a replicate operation by its UUID.

        Args:
            uuid: The ID of the replicate operation.

        Returns:
            None
        """
        return executor.execute(
            response_callback=lambda _: None,
            method=self._connection.post,
            weaviate_object={},
            path=f"/replication/replicate/{uuid}/cancel",
            status_codes=_ExpectedStatusCodes(204, "replicate cancel"),
            error_msg="Failed to cancel replicate operation",
        )

    def delete(
        self,
        *,
        uuid: UUID,
    ) -> executor.Result[None]:
        """Delete a replicate operation by its UUID.

        Args:
            uuid: The ID of the replicate operation.

        Returns:
            None
        """
        return executor.execute(
            response_callback=lambda _: None,
            method=self._connection.delete,
            path=f"/replication/replicate/{uuid}",
            status_codes=_ExpectedStatusCodes(204, "replicate delete"),
            error_msg="Failed to delete replicate operation",
        )

    def delete_all(self) -> executor.Result[None]:
        """Delete all replicate operations.

        Returns:
            None
        """
        return executor.execute(
            response_callback=lambda _: None,
            method=self._connection.delete,
            path="/replication/replicate",
            status_codes=_ExpectedStatusCodes(204, "replicate delete all"),
            error_msg="Failed to delete all replicate operations",
        )



from weaviate.collections.config.executor import _ConfigCollectionExecutor
from weaviate.connect import executor
from weaviate.connect.v4 import ConnectionSync


@executor.wrap("sync")
class _ConfigCollection(_ConfigCollectionExecutor[ConnectionSync]):
    pass



from weaviate.rbac.models import (
    AliasPermissionOutput,
    BackupsPermissionOutput,
    ClusterPermissionOutput,
    CollectionsPermissionOutput,
    DataPermissionOutput,
    GroupAssignment,
    GroupsPermissionOutput,
    NodesPermissionOutput,
    PermissionsOutputType,
    ReplicatePermissionOutput,
    RoleScope,
    RolesPermissionOutput,
    TenantsPermissionOutput,
    UserAssignment,
    UsersPermissionOutput,
)

__all__ = [
    "PermissionsOutputType",
    "BackupsPermissionOutput",
    "ClusterPermissionOutput",
    "CollectionsPermissionOutput",
    "DataPermissionOutput",
    "NodesPermissionOutput",
    "RolesPermissionOutput",
    "RoleScope",
    "UsersPermissionOutput",
    "UserAssignment",
    "AliasPermissionOutput",
    "ReplicatePermissionOutput",
    "GroupsPermissionOutput",
    "TenantsPermissionOutput",
    "GroupAssignment",
]



from weaviate.backup.backup import (
    BackupReturn,
    BackupStatus,
    BackupStatusReturn,
    BackupStorage,
)

__all__ = [
    "BackupStatus",
    "BackupStatusReturn",
    "BackupStorage",
    "BackupReturn",
]
